@@ -29,13 +29,14 @@
 ** Configuration transfers occur in groups.  These are the allowed
 ** groupings:
 */
-#define CONFIGSET_SKIN      0x000001     /* WWW interface appearance */
-#define CONFIGSET_TKT       0x000002     /* Ticket configuration */
-#define CONFIGSET_PROJ      0x000004     /* Project name */
-#define CONFIGSET_SHUN      0x000008     /* Shun settings */
-#define CONFIGSET_USER      0x000010     /* The USER table */
-#define CONFIGSET_ADDR      0x000020     /* The CONCEALED table */
-#define CONFIGSET_XFER      0x000040     /* Transfer configuration */
+#define CONFIGSET_CSS       0x000001     /* Style sheet only */
+#define CONFIGSET_SKIN      0x000002     /* WWW interface appearance */
+#define CONFIGSET_TKT       0x000004     /* Ticket configuration */
+#define CONFIGSET_PROJ      0x000008     /* Project name */
+#define CONFIGSET_SHUN      0x000010     /* Shun settings */
+#define CONFIGSET_USER      0x000020     /* The USER table */
+#define CONFIGSET_ADDR      0x000040     /* The CONCEALED table */
+#define CONFIGSET_XFER      0x000080     /* Transfer configuration */
 
 #define CONFIGSET_ALL       0x0000ff     /* Everything */
 
@@ -54,7 +55,9 @@ static struct {
 } aGroupName[] = {
   { "/email",        CONFIGSET_ADDR,  "Concealed email addresses in tickets" },
   { "/project",      CONFIGSET_PROJ,  "Project name and description"         },
-  { "/skin",         CONFIGSET_SKIN,  "Web interface apparance settings"     },
+  { "/skin",         CONFIGSET_SKIN | CONFIGSET_CSS,
+                                      "Web interface apparance settings"     },
+  { "/css",          CONFIGSET_CSS,   "Style sheet"                          },
   { "/shun",         CONFIGSET_SHUN,  "List of shunned artifacts"            },
   { "/ticket",       CONFIGSET_TKT,   "Ticket setup",                        },
   { "/user",         CONFIGSET_USER,  "Users and privilege settings"         },
@@ -65,7 +68,7 @@ static struct {
 
 /*
 ** The following is a list of settings that we are willing to
-** transfer.  
+** transfer.
 **
 ** Setting names that begin with an alphabetic characters refer to
 ** single entries in the CONFIG table.  Setting names that begin with
@@ -75,24 +78,34 @@ static struct {
   const char *zName;   /* Name of the configuration parameter */
   int groupMask;       /* Which config groups is it part of */
 } aConfig[] = {
-  { "css",                    CONFIGSET_SKIN },
+  { "css",                    CONFIGSET_CSS  },
   { "header",                 CONFIGSET_SKIN },
   { "footer",                 CONFIGSET_SKIN },
   { "logo-mimetype",          CONFIGSET_SKIN },
   { "logo-image",             CONFIGSET_SKIN },
+  { "background-mimetype",    CONFIGSET_SKIN },
+  { "background-image",       CONFIGSET_SKIN },
+  { "index-page",             CONFIGSET_SKIN },
+  { "timeline-block-markup",  CONFIGSET_SKIN },
+  { "timeline-max-comment",   CONFIGSET_SKIN },
+  { "adunit",                 CONFIGSET_SKIN },
+  { "adunit-omit-if-admin",   CONFIGSET_SKIN },
+  { "adunit-omit-if-user",    CONFIGSET_SKIN },
+
+#ifdef FOSSIL_ENABLE_TCL
+  { "tcl",                    CONFIGSET_SKIN|CONFIGSET_TKT|CONFIGSET_XFER },
+  { "tcl-setup",              CONFIGSET_SKIN|CONFIGSET_TKT|CONFIGSET_XFER },
+#endif
+
   { "project-name",           CONFIGSET_PROJ },
   { "project-description",    CONFIGSET_PROJ },
   { "manifest",               CONFIGSET_PROJ },
+  { "binary-glob",            CONFIGSET_PROJ },
   { "ignore-glob",            CONFIGSET_PROJ },
   { "crnl-glob",              CONFIGSET_PROJ },
   { "empty-dirs",             CONFIGSET_PROJ },
   { "allow-symlinks",         CONFIGSET_PROJ },
-  { "index-page",             CONFIGSET_SKIN },
-#ifdef FOSSIL_ENABLE_TCL
-  { "tcl",                    CONFIGSET_SKIN|CONFIGSET_TKT|CONFIGSET_XFER },
-#endif
-  { "timeline-block-markup",  CONFIGSET_SKIN },
-  { "timeline-max-comment",   CONFIGSET_SKIN },
+
   { "ticket-table",           CONFIGSET_TKT  },
   { "ticket-common",          CONFIGSET_TKT  },
   { "ticket-change",          CONFIGSET_TKT  },
@@ -105,11 +118,16 @@ static struct {
   { "ticket-title-expr",      CONFIGSET_TKT  },
   { "ticket-closed-expr",     CONFIGSET_TKT  },
   { "@reportfmt",             CONFIGSET_TKT  },
+
   { "@user",                  CONFIGSET_USER },
+
   { "@concealed",             CONFIGSET_ADDR },
+
   { "@shun",                  CONFIGSET_SHUN },
+
   { "xfer-common-script",     CONFIGSET_XFER },
   { "xfer-push-script",       CONFIGSET_XFER },
+
 };
 static int iConfig = 0;
 
@@ -145,6 +163,28 @@ const char *configure_next_name(int iMask){
   return 0;
 }
 
+/*
+** Return a pointer to a string that contains the RHS of an IN operator
+** that will select CONFIG table names that are part of the configuration
+** that matchines iMatch.
+*/
+const char *configure_inop_rhs(int iMask){
+  Blob x;
+  int i;
+  const char *zSep = "";
+
+  blob_zero(&x);
+  blob_append(&x, "(", 1);
+  for(i=0; i<count(aConfig); i++){
+    if( (aConfig[i].groupMask & iMask)==0 ) continue;
+    if( aConfig[i].zName[0]=='@' ) continue;
+    blob_appendf(&x, "%s'%s'", zSep, aConfig[i].zName);
+    zSep = ",";
+  }
+  blob_append(&x, ")", 1);
+  return blob_str(&x);
+}
+
 /*
 ** Return the mask for the named configuration parameter if it can be
 ** safely exported.  Return 0 if the parameter is not safe to export.
@@ -187,7 +227,7 @@ void configure_render_special_name(const char *zName, Blob *pOut){
   if( fossil_strcmp(zName, "@shun")==0 ){
     db_prepare(&q, "SELECT uuid FROM shun");
     while( db_step(&q)==SQLITE_ROW ){
-      blob_appendf(pOut, "INSERT OR IGNORE INTO shun VALUES('%s');\n", 
+      blob_appendf(pOut, "INSERT OR IGNORE INTO shun VALUES('%s');\n",
         db_column_text(&q, 0)
       );
     }
@@ -196,7 +236,7 @@ void configure_render_special_name(const char *zName, Blob *pOut){
     db_prepare(&q, "SELECT title, cols, sqlcode FROM reportfmt");
     while( db_step(&q)==SQLITE_ROW ){
       blob_appendf(pOut, "INSERT INTO _xfer_reportfmt(title,cols,sqlcode)"
-                         " VALUES(%Q,%Q,%Q);\n", 
+                         " VALUES(%Q,%Q,%Q);\n",
         db_column_text(&q, 0),
         db_column_text(&q, 1),
         db_column_text(&q, 2)
@@ -204,7 +244,7 @@ void configure_render_special_name(const char *zName, Blob *pOut){
     }
     db_finalize(&q);
   }else if( fossil_strcmp(zName, "@user")==0 ){
-    db_prepare(&q, 
+    db_prepare(&q,
         "SELECT login, CASE WHEN length(pw)==40 THEN pw END,"
         "       cap, info, quote(photo) FROM user");
     while( db_step(&q)==SQLITE_ROW ){
@@ -239,7 +279,7 @@ void configure_render_special_name(const char *zName, Blob *pOut){
 **
 ** The config_is_reset() function takes the integer valued argument and
 ** ANDs it against the static variable "configHasBeenReset" below.  The
-** function returns TRUE or FALSE depending on the result depending on 
+** function returns TRUE or FALSE depending on the result depending on
 ** whether or not the corresponding configuration table has been reset.  The
 ** config_reset() function adds the bits to "configHasBeenReset" that
 ** are given in the argument.
@@ -302,7 +342,7 @@ void configure_prepare_to_receive(int replaceFlag){
     @    SELECT uid,login,pw,cap,cookie,ipaddr,cexpire,info,photo FROM user;
   ;
   db_multi_exec(zSQL1);
-  
+
   /* When the replace flag is set, add triggers that run the first time
   ** that new data is seen.  The triggers run only once and delete all the
   ** existing data.
@@ -406,7 +446,7 @@ static int safeInt(const char *z){
 ** The second token is a primary key for the table identified by zName.  If
 ** The entry with the corresponding primary key exists and has a more recent
 ** mtime, then nothing happens.  If the entry does not exist or if it has
-** an older mtime, then the content described by subsequent token pairs is 
+** an older mtime, then the content described by subsequent token pairs is
 ** inserted.  The first element of each token pair is a column name and
 ** the second is its value.
 **
@@ -484,7 +524,7 @@ void configure_receive(const char *zName, Blob *pContent, int groupMask){
       thisMask = configure_is_exportable(aType[ii].zName);
     }
     if( (thisMask & groupMask)==0 ) return;
-    
+
     blob_zero(&sql);
     if( groupMask & CONFIGSET_OVERWRITE ){
       if( (thisMask & configHasBeenReset)==0 && aType[ii].zName[0]!='/' ){
@@ -571,7 +611,7 @@ void configure_receive_all(Blob *pIn, int groupMask){
     }
   }
 }
-    
+
 
 /*
 ** Send "config" cards using the new format for all elements of a group
@@ -721,7 +761,7 @@ static void export_config(
 ){
   Blob out;
   blob_zero(&out);
-  blob_appendf(&out, 
+  blob_appendf(&out,
     "# The \"%s\" configuration exported from\n"
     "# repository \"%s\"\n"
     "# on %s\n",
@@ -782,7 +822,7 @@ static void export_config(
 **    %fossil configuration sync AREA ?URL?
 **
 **         Synchronize configuration changes in the local repository with
-**         the remote repository at URL.  
+**         the remote repository at URL.
 **
 ** Options:
 **    -R|--repository FILE       Extract info from repository FILE
@@ -817,7 +857,7 @@ void configuration_cmd(void){
     }
     export_config(mask, g.argv[3], iStart, g.argv[4]);
   }else
-  if( strncmp(zMethod, "import", n)==0 
+  if( strncmp(zMethod, "import", n)==0
        || strncmp(zMethod, "merge", n)==0 ){
     Blob in;
     int groupMask;
@@ -880,7 +920,7 @@ void configuration_cmd(void){
     char *zBackup;
     if( g.argc!=4 ) usage("reset AREA");
     mask = configure_name_to_mask(g.argv[3], 1);
-    zBackup = db_text(0, 
+    zBackup = db_text(0,
        "SELECT strftime('config-backup-%%Y%%m%%d%%H%%M%%f','now')");
     db_begin_transaction();
     export_config(mask, g.argv[3], 0, zBackup);
@@ -902,8 +942,8 @@ void configuration_cmd(void){
     }
     db_end_transaction(0);
     fossil_print("Configuration reset to factory defaults.\n");
-    fossil_print("To recover, use:  %s %s import %s\n", 
-            fossil_nameofexe(), g.argv[1], zBackup);
+    fossil_print("To recover, use:  %s %s import %s\n",
+            g.argv[0], g.argv[1], zBackup);
   }else
   {
     fossil_fatal("METHOD should be one of:"
