@@ -67,7 +67,7 @@ static void db_err(const char *zFormat, ...){
   va_list ap;
   char *z;
   int rc = 1;
-  static const char zRebuildMsg[] = 
+  static const char zRebuildMsg[] =
       "If you have recently updated your fossil executable, you might\n"
       "need to run \"fossil all rebuild\" to bring the repository\n"
       "schemas up to date.\n";
@@ -94,7 +94,7 @@ static void db_err(const char *zFormat, ...){
                "<pre>%h</pre><p>%s</p>", z, zRebuildMsg);
     cgi_reply();
   }else{
-    fprintf(stderr, "%s: %s\n\n%s", fossil_nameofexe(), z, zRebuildMsg);
+    fprintf(stderr, "%s: %s\n\n%s", g.argv[0], z, zRebuildMsg);
   }
   free(z);
   db_force_rollback();
@@ -117,6 +117,9 @@ static struct DbLocalData {
     int sequence;               /* Call functions in sequence order */
   } aHook[5];
   char *azDeleteOnFail[3];  /* Files to delete on a failure */
+  char *azBeforeCommit[5];  /* Commands to run prior to COMMIT */
+  int nBeforeCommit;        /* Number of entries in azBeforeCommit */
+  int nPriorChanges;        /* sqlite3_total_changes() at transaction start */
 } db = {0, 0, 0, 0, 0, 0, };
 
 /*
@@ -151,6 +154,7 @@ void db_begin_transaction(void){
   if( db.nBegin==0 ){
     db_multi_exec("BEGIN");
     sqlite3_commit_hook(g.db, db_verify_at_commit, 0);
+    db.nPriorChanges = sqlite3_total_changes(g.db);
   }
   db.nBegin++;
 }
@@ -161,7 +165,14 @@ void db_end_transaction(int rollbackFlag){
   db.nBegin--;
   if( db.nBegin==0 ){
     int i;
-    if( db.doRollback==0 ) leaf_do_pending_checks();
+    if( db.doRollback==0 && db.nPriorChanges<sqlite3_total_changes(g.db) ){
+      while( db.nBeforeCommit ){
+        db.nBeforeCommit--;
+        sqlite3_exec(g.db, db.azBeforeCommit[db.nBeforeCommit], 0, 0, 0);
+        sqlite3_free(db.azBeforeCommit[db.nBeforeCommit]);
+      }
+      leaf_do_pending_checks();
+    }
     for(i=0; db.doRollback==0 && i<db.nCommitHook; i++){
       db.doRollback |= db.aHook[i].xHook();
     }
@@ -490,6 +501,20 @@ int db_multi_exec(const char *zSql, ...){
   return rc;
 }
 
+/*
+** Optionally make the following changes to the database if feasible and
+** convenient.  Do not start a transaction for these changes, but only
+** make these changes if other changes are also being made.
+*/
+void db_optional_sql(const char *zDb, const char *zSql, ...){
+  if( db_is_writeable(zDb) && db.nBeforeCommit < count(db.azBeforeCommit) ){
+    va_list ap;
+    va_start(ap, zSql);
+    db.azBeforeCommit[db.nBeforeCommit++] = sqlite3_vmprintf(zSql, ap);
+    va_end(ap);
+  }
+}
+
 /*
 ** Execute a query and return a single integer value.
 */
@@ -653,6 +678,22 @@ void db_now_function(
   sqlite3_result_int64(context, time(0));
 }
 
+/*
+** Function to return the check-in time for a file.
+*/
+void db_checkin_mtime_function(
+  sqlite3_context *context,
+  int argc,
+  sqlite3_value **argv
+){
+  i64 mtime;
+  int rc = mtime_of_manifest_file(sqlite3_value_int(argv[0]),
+                                  sqlite3_value_int(argv[1]), &mtime);
+  if( rc==0 ){
+    sqlite3_result_int64(context, mtime);
+  }
+}
+
 
 /*
 ** Open a database file.  Return a pointer to the new database
@@ -672,25 +713,42 @@ static sqlite3 *openDatabase(const char *zDbName){
   if( rc!=SQLITE_OK ){
     db_err(sqlite3_errmsg(db));
   }
-  sqlite3_busy_timeout(db, 5000); 
+  sqlite3_busy_timeout(db, 5000);
   sqlite3_wal_autocheckpoint(db, 1);  /* Set to checkpoint frequently */
   sqlite3_create_function(db, "now", 0, SQLITE_ANY, 0, db_now_function, 0, 0);
+  sqlite3_create_function(db, "checkin_mtime", 2, SQLITE_ANY, 0,
+                          db_checkin_mtime_function, 0, 0);
   return db;
 }
 
 
+/*
+** Detaches the zLabel database.
+*/
+void db_detach(const char *zLabel){
+  db_multi_exec("DETACH DATABASE %s", zLabel);
+}
+
+/*
+** zDbName is the name of a database file.  Attach zDbName using
+** the name zLabel.
+*/
+void db_attach(const char *zDbName, const char *zLabel){
+  db_multi_exec("ATTACH DATABASE %Q AS %s", zDbName, zLabel);
+}
+
 /*
 ** zDbName is the name of a database file.  If no other database
 ** file is open, then open this one.  If another database file is
 ** already open, then attach zDbName using the name zLabel.
 */
-static void db_open_or_attach(const char *zDbName, const char *zLabel){
+void db_open_or_attach(const char *zDbName, const char *zLabel){
   if( !g.db ){
     g.db = openDatabase(zDbName);
     g.zMainDbType = zLabel;
     db_connection_init();
   }else{
-    db_multi_exec("ATTACH DATABASE %Q AS %s", zDbName, zLabel);
+    db_attach(zDbName, zLabel);
   }
 }
 
@@ -794,7 +852,7 @@ static int isValidLocalDb(const char *zDbName){
 
   /* If the "isexe" column is missing from the vfile table, then
   ** add it now.   This code added on 2010-03-06.  After all users have
-  ** upgraded, this code can be safely deleted. 
+  ** upgraded, this code can be safely deleted.
   */
   if( !db_local_column_exists("vfile", "isexe") ){
     db_multi_exec("ALTER TABLE vfile ADD COLUMN isexe BOOLEAN DEFAULT 0");
@@ -802,12 +860,12 @@ static int isValidLocalDb(const char *zDbName){
 
   /* If "islink"/"isLink" columns are missing from tables, then
   ** add them now.   This code added on 2011-01-17 and 2011-08-27.
-  ** After all users have upgraded, this code can be safely deleted. 
+  ** After all users have upgraded, this code can be safely deleted.
   */
   if( !db_local_column_exists("vfile", "islink") ){
     db_multi_exec("ALTER TABLE vfile ADD COLUMN islink BOOLEAN DEFAULT 0");
   }
-  
+
   if( !db_local_column_exists("stashfile", "isLink") &&
        db_local_table_exists("stashfile") ){
     db_multi_exec("ALTER TABLE stashfile ADD COLUMN isLink BOOLEAN DEFAULT 0");
@@ -817,7 +875,7 @@ static int isValidLocalDb(const char *zDbName){
        db_local_table_exists("undo") ){
     db_multi_exec("ALTER TABLE undo ADD COLUMN isLink BOOLEAN DEFAULT 0");
   }
-  
+
   if( !db_local_column_exists("undo_vfile", "islink") &&
        db_local_table_exists("undo_vfile") ){
     db_multi_exec("ALTER TABLE undo_vfile ADD COLUMN islink BOOLEAN DEFAULT 0");
@@ -833,7 +891,7 @@ static int isValidLocalDb(const char *zDbName){
 ** For legacy, also look for ".fos".  The use of ".fos" is deprecated
 ** since "fos" has negative connotations in Hungarian, we are told.
 **
-** If no valid _FOSSIL_ or .fos file is found, we move up one level and 
+** If no valid _FOSSIL_ or .fos file is found, we move up one level and
 ** try again. Once the file is found, the g.zLocalRoot variable is set
 ** to the root of the repository tree and this routine returns 1.  If
 ** no database is found, then this routine return 0.
@@ -845,8 +903,8 @@ static int isValidLocalDb(const char *zDbName){
 int db_open_local(void){
   int i, n;
   char zPwd[2000];
-  static const char *aDbName[] = { "/_FOSSIL_", "/.fslckout", "/.fos" };
-  
+  static const char *const aDbName[] = { "/_FOSSIL_", "/.fslckout", "/.fos" };
+
   if( g.localOpen) return 1;
   file_getcwd(zPwd, sizeof(zPwd)-20);
   n = strlen(zPwd);
@@ -976,7 +1034,11 @@ rep_not_found:
 #ifdef FOSSIL_ENABLE_JSON
     g.json.resultCode = FSL_JSON_E_DB_NOT_FOUND;
 #endif
-    fossil_fatal("use --repository or -R to specify the repository database");
+    if( nArgUsed==0 ){
+      fossil_fatal("use --repository or -R to specify the repository database");
+    }else{
+      fossil_fatal("specify the repository name as a command-line argument");
+    }
   }
 }
 
@@ -1000,6 +1062,13 @@ int db_schema_is_outofdate(void){
                    "   AND value<>'%s'", AUX_SCHEMA);
 }
 
+/*
+** Return true if the database is writeable
+*/
+int db_is_writeable(const char *zName){
+  return g.db!=0 && !sqlite3_db_readonly(g.db, db_name(zName));
+}
+
 /*
 ** Verify that the repository schema is correct.  If it is not correct,
 ** issue a fatal error and die.
@@ -1037,7 +1106,7 @@ void move_repo_cmd(void){
     fossil_fatal("not in a local checkout");
     return;
   }
-  file_canonical_name(g.argv[2], &repo);
+  file_canonical_name(g.argv[2], &repo, 0);
   zRepo = blob_str(&repo);
   if( file_access(zRepo, 0) ){
     fossil_fatal("no such file: %s", zRepo);
@@ -1053,7 +1122,7 @@ void move_repo_cmd(void){
 */
 void db_must_be_within_tree(void){
   if( db_open_local()==0 ){
-    fossil_fatal("not within an open checkout");
+    fossil_fatal("current directory is not within an open checkout");
   }
   db_open_repository(0);
   db_verify_schema();
@@ -1173,18 +1242,46 @@ void db_create_default_users(int setupUserOnly, const char *zDefaultUser){
   }
 }
 
+/*
+** Return a pointer to a string that contains the RHS of an IN operator
+** that will select CONFIG table names that are in the list of control
+** settings.
+*/
+const char *db_setting_inop_rhs(){
+  Blob x;
+  int i;
+  const char *zSep = "";
+
+  blob_zero(&x);
+  blob_append(&x, "(", 1);
+  for(i=0; ctrlSettings[i].name; i++){
+    blob_appendf(&x, "%s'%s'", zSep, ctrlSettings[i].name);
+    zSep = ",";
+  }
+  blob_append(&x, ")", 1);
+  return blob_str(&x);
+}
+
 /*
 ** Fill an empty repository database with the basic information for a
 ** repository. This function is shared between 'create_repository_cmd'
 ** ('new') and 'reconstruct_cmd' ('reconstruct'), both of which create
 ** new repositories.
 **
+** The zTemplate parameter determines if the settings for the repository
+** should be copied from another repository.  If zTemplate is 0 then the
+** settings will have their normal default values.  If zTemplate is
+** non-zero, it is assumed that the caller of this function has already
+** attached a database using the label "settingSrc".  If not, the call to
+** this function will fail.
+**
 ** The zInitialDate parameter determines the date of the initial check-in
 ** that is automatically created.  If zInitialDate is 0 then no initial
 ** check-in is created. The makeServerCodes flag determines whether or
 ** not server and project codes are invented for this repository.
 */
 void db_initial_setup(
+  const char *zTemplate,       /* Repository from which to copy settings. */
   const char *zInitialDate,    /* Initial date of repository. (ex: "now") */
   const char *zDefaultUser,    /* Default user for the repository */
   int makeServerCodes          /* True to make new server & project codes */
@@ -1208,6 +1305,43 @@ void db_initial_setup(
   db_create_default_users(0, zDefaultUser);
   user_select();
 
+  if( zTemplate ){
+    /*
+    ** Copy all settings from the supplied template repository.
+    */
+    db_multi_exec(
+      "INSERT OR REPLACE INTO config"
+      " SELECT name,value,mtime FROM settingSrc.config"
+      "  WHERE (name IN %s OR name IN %s)"
+      "    AND name NOT GLOB 'project-*';",
+      configure_inop_rhs(CONFIGSET_ALL),
+      db_setting_inop_rhs()
+    );
+    db_multi_exec(
+      "REPLACE INTO reportfmt SELECT * FROM settingSrc.reportfmt;"
+    );
+
+    /*
+    ** Copy the user permissions, contact information, last modified
+    ** time, and photo for all the "system" users from the supplied
+    ** template repository into the one being setup.  The other columns
+    ** are not copied because they contain security information or other
+    ** data specific to the other repository.  The list of columns copied
+    ** by this SQL statement may need to be revised in the future.
+    */
+    db_multi_exec("UPDATE user SET"
+      "  cap = (SELECT u2.cap FROM settingSrc.user u2"
+      "         WHERE u2.login = user.login),"
+      "  info = (SELECT u2.info FROM settingSrc.user u2"
+      "          WHERE u2.login = user.login),"
+      "  mtime = (SELECT u2.mtime FROM settingSrc.user u2"
+      "           WHERE u2.login = user.login),"
+      "  photo = (SELECT u2.photo FROM settingSrc.user u2"
+      "           WHERE u2.login = user.login)"
+      " WHERE user.login IN ('anonymous','nobody','developer','reader');"
+    );
+  }
+
   if( zInitialDate ){
     int rid;
     blob_zero(&manifest);
@@ -1243,7 +1377,17 @@ void db_initial_setup(
 ** admin user. This can be overridden using the -A|--admin-user
 ** parameter.
 **
+** By default, all settings will be initialized to their default values.
+** This can be overridden using the --template parameter to specify a
+** repository file from which to copy the initial settings.  When a template
+** repository is used, almost all of the settings accessible from the setup
+** page, either directly or indirectly, will be copied.  Normal users and
+** their associated permissions will not be copied; however, the system
+** default users "anonymous", "nobody", "reader", "developer", and their
+** associated permissions will be copied.
+**
 ** Options:
+**    --template      FILE      copy settings from repository file
 **    --admin-user|-A USERNAME  select given USERNAME as admin user
 **    --date-override DATETIME  use DATETIME as time of the initial checkin
 **
@@ -1251,9 +1395,11 @@ void db_initial_setup(
 */
 void create_repository_cmd(void){
   char *zPassword;
+  const char *zTemplate;      /* Repository from which to copy settings */
   const char *zDate;          /* Date of the initial check-in */
   const char *zDefaultUser;   /* Optional name of the default user */
 
+  zTemplate = find_option("template",0,1);
   zDate = find_option("date-override",0,1);
   zDefaultUser = find_option("admin-user","A",1);
   if( zDate==0 ) zDate = "now";
@@ -1263,13 +1409,15 @@ void create_repository_cmd(void){
   db_create_repository(g.argv[2]);
   db_open_repository(g.argv[2]);
   db_open_config(0);
+  if( zTemplate ) db_attach(zTemplate, "settingSrc");
   db_begin_transaction();
-  db_initial_setup(zDate, zDefaultUser, 1);
+  db_initial_setup(zTemplate, zDate, zDefaultUser, 1);
   db_end_transaction(0);
+  if( zTemplate ) db_detach("settingSrc");
   fossil_print("project-id: %s\n", db_get("project-code", 0));
   fossil_print("server-id:  %s\n", db_get("server-code", 0));
   zPassword = db_text(0, "SELECT pw FROM user WHERE login=%Q", g.zLogin);
-  fossil_print("admin-user: %s (initial password is \"%s\")\n", 
+  fossil_print("admin-user: %s (initial password is \"%s\")\n",
                g.zLogin, zPassword);
 }
 
@@ -1315,7 +1463,7 @@ static void db_sql_user(
 
 /*
 ** Implement the cgi() SQL function.  cgi() takes a an argument which is
-** a name of CGI query parameter. The value of that parameter is returned, 
+** a name of CGI query parameter. The value of that parameter is returned,
 ** if available. optional second argument will be returned if the first
 ** doesn't exist as a CGI parameter.
 */
@@ -1332,33 +1480,51 @@ static void db_sql_cgi(sqlite3_context *context, int argc, sqlite3_value **argv)
 }
 
 /*
-** This is used by the [commit] command.
+** SQL function:
 **
-** Return true if either:
+**       is_selected(id)
+**       if_selected(id, X, Y)
 **
-**     a) Global.aCommitFile is NULL, or
-**     b) Global.aCommitFile contains the integer passed as an argument.
+** On the commit command, when filenames are specified (in order to do
+** a partial commit) the vfile.id values for the named files are loaded
+** into the g.aCommitFile[] array.  This function looks at that array
+** to see if a file is named on the command-line.
 **
-** Otherwise return false.
+** In the first form (1 argument) return TRUE if either no files are
+** named on the command line (g.aCommitFile is NULL meaning that all
+** changes are to be committed) or if id is found in g.aCommitFile[]
+** (meaning that id was named on the command-line).
+**
+** In the second form (3 arguments) return argument X if true and Y
+** if false.  Except if Y is NULL then always return X.
 */
 static void file_is_selected(
   sqlite3_context *context,
   int argc,
   sqlite3_value **argv
 ){
-  assert(argc==1);
+  int rc = 0;
+
+  assert(argc==1 || argc==3);
   if( g.aCommitFile ){
     int iId = sqlite3_value_int(argv[0]);
     int ii;
     for(ii=0; g.aCommitFile[ii]; ii++){
       if( iId==g.aCommitFile[ii] ){
-        sqlite3_result_int(context, 1);
-        return;
+        rc = 1;
+        break;
       }
     }
-    sqlite3_result_int(context, 0);
   }else{
-    sqlite3_result_int(context, 1);
+    rc = 1;
+  }
+  if( argc==1 ){
+    sqlite3_result_int(context, rc);
+  }else{
+    assert( argc==3 );
+    assert( rc==0 || rc==1 );
+    if( sqlite3_value_type(argv[2-rc])==SQLITE_NULL ) rc = 1-rc;
+    sqlite3_result_value(context, argv[2-rc]);
   }
 }
 
@@ -1431,7 +1597,10 @@ LOCAL void db_connection_init(void){
   sqlite3_create_function(g.db, "cgi", 2, SQLITE_ANY, 0, db_sql_cgi, 0, 0);
   sqlite3_create_function(g.db, "print", -1, SQLITE_UTF8, 0,db_sql_print,0,0);
   sqlite3_create_function(
-    g.db, "file_is_selected", 1, SQLITE_UTF8, 0, file_is_selected,0,0
+    g.db, "is_selected", 1, SQLITE_UTF8, 0, file_is_selected,0,0
+  );
+  sqlite3_create_function(
+    g.db, "if_selected", 3, SQLITE_UTF8, 0, file_is_selected,0,0
   );
   if( g.fSqlTrace ){
     sqlite3_trace(g.db, db_sql_trace, 0);
@@ -1442,7 +1611,7 @@ LOCAL void db_connection_init(void){
 ** Return true if the string zVal represents "true" (or "false").
 */
 int is_truth(const char *zVal){
-  static const char *azOn[] = { "on", "yes", "true", "1" };
+  static const char *const azOn[] = { "on", "yes", "true", "1" };
   int i;
   for(i=0; i<sizeof(azOn)/sizeof(azOn[0]); i++){
     if( fossil_stricmp(zVal,azOn[i])==0 ) return 1;
@@ -1450,7 +1619,7 @@ int is_truth(const char *zVal){
   return 0;
 }
 int is_false(const char *zVal){
-  static const char *azOff[] = { "off", "no", "false", "0" };
+  static const char *const azOff[] = { "off", "no", "false", "0" };
   int i;
   for(i=0; i<sizeof(azOff)/sizeof(azOff[0]); i++){
     if( fossil_stricmp(zVal,azOff[i])==0 ) return 1;
@@ -1489,7 +1658,7 @@ static char *db_get_do_versionable(const char *zName, char *zNonVersionedSetting
     const char *zName, *zValue;
   } *cacheEntry = 0;
   static struct _cacheEntry *cache = 0;
-  
+
   /* Look up name in cache */
   cacheEntry = cache;
   while( cacheEntry!=0 ){
@@ -1700,7 +1869,7 @@ void db_record_repository_filename(const char *zName){
     if( !g.localOpen ) return;
     zName = db_repository_filename();
   }
-  file_canonical_name(zName, &full);
+  file_canonical_name(zName, &full, 0);
   db_swap_connections();
   db_multi_exec(
      "INSERT OR IGNORE INTO global_config(name,value)"
@@ -1708,13 +1877,23 @@ void db_record_repository_filename(const char *zName){
      blob_str(&full)
   );
   if( g.localOpen && g.zLocalRoot && g.zLocalRoot[0] ){
+    Blob localRoot;
+    file_canonical_name(g.zLocalRoot, &localRoot, 1);
     db_multi_exec(
       "REPLACE INTO global_config(name, value)"
       "VALUES('ckout:%q','%q');",
-      g.zLocalRoot, blob_str(&full)
+      blob_str(&localRoot), blob_str(&full)
     );
+    db_swap_connections();
+    db_optional_sql("repository",
+        "REPLACE INTO config(name,value,mtime)"
+        "VALUES('ckout:%q',1,now())",
+        blob_str(&localRoot)
+    );
+    blob_reset(&localRoot);
+  }else{
+    db_swap_connections();
   }
-  db_swap_connections();
   blob_reset(&full);
 }
 
@@ -1751,10 +1930,15 @@ void cmd_open(void){
   if( !allowNested && db_open_local() ){
     fossil_panic("already within an open tree rooted at %s", g.zLocalRoot);
   }
-  file_canonical_name(g.argv[2], &path);
+  file_canonical_name(g.argv[2], &path, 0);
   db_open_repository(blob_str(&path));
-  db_init_database("./_FOSSIL_", zLocalSchema, (char*)0);
-  db_delete_on_failure("./_FOSSIL_");
+#if defined(_WIN32)
+# define LOCALDB_NAME "./_FOSSIL_"
+#else
+# define LOCALDB_NAME "./.fslckout"
+#endif
+  db_init_database(LOCALDB_NAME, zLocalSchema, (char*)0);
+  db_delete_on_failure(LOCALDB_NAME);
   db_open_local();
   db_lset("repository", g.argv[2]);
   db_record_repository_filename(blob_str(&path));
@@ -1842,6 +2026,7 @@ struct stControlSettings const ctrlSettings[] = {
   { "access-log",    0,                0, 0, "off"                 },
   { "allow-symlinks",0,                0, 1, "off"                 },
   { "auto-captcha",  "autocaptcha",    0, 0, "on"                  },
+  { "auto-hyperlink",0,                0, 0, "on",                 },
   { "auto-shun",     0,                0, 0, "on"                  },
   { "autosync",      0,                0, 0, "on"                  },
   { "binary-glob",   0,               32, 1, ""                    },
@@ -1849,6 +2034,7 @@ struct stControlSettings const ctrlSettings[] = {
   { "case-sensitive",0,                0, 0, "on"                  },
   { "crnl-glob",     0,               16, 1, ""                    },
   { "default-perms", 0,               16, 0, "u"                   },
+  { "diff-binary",   0,                0, 0, "on"                  },
   { "diff-command",  0,               16, 0, ""                    },
   { "dont-push",     0,                0, 0, "off"                 },
   { "editor",        0,               16, 0, ""                    },
@@ -1873,6 +2059,7 @@ struct stControlSettings const ctrlSettings[] = {
   { "ssh-command",   0,               32, 0, ""                    },
 #ifdef FOSSIL_ENABLE_TCL
   { "tcl",           0,                0, 0, "off"                 },
+  { "tcl-setup",     0,               40, 0, ""                    },
 #endif
   { "web-browser",   0,               32, 0, ""                    },
   { "white-foreground", 0,             0, 0, "off"                 },
@@ -1899,13 +2086,18 @@ struct stControlSettings const ctrlSettings[] = {
 **
 **    allow-symlinks   If enabled, don't follow symlinks, and instead treat
 **     (versionable)   them as symlinks on Unix. Has no effect on Windows
-**                     (existing links in repository created on Unix become 
+**                     (existing links in repository created on Unix become
 **                     plain-text files with link destination path inside).
 **                     Default: off
 **
 **    auto-captcha     If enabled, the Login page provides a button to
 **                     fill in the captcha password.  Default: on
 **
+**    auto-hyperlink   Use javascript to enable hyperlinks on web pages
+**                     for all users (regardless of the "h" privilege) if the
+**                     User-Agent string in the HTTP header look like it came
+**                     from real person, not a spider or bot.  Default: on
+**
 **    auto-shun        If enabled, automatically pull the shunning list
 **                     from a server to which the client autosyncs.
 **                     Default: on
@@ -1937,6 +2129,10 @@ struct stControlSettings const ctrlSettings[] = {
 **                     information on permissions see Users page in Server
 **                     Administration of the HTTP UI. Default: u.
 **
+**    diff-binary      If TRUE (the default), permit files that may be binary
+**                     or that match the "binary-glob" setting to be used with
+**                     external diff programs.  If FALSE, skip these files.
+**
 **    diff-command     External command to run when performing a diff.
 **                     If undefined, the internal text diff will be used.
 **
@@ -2028,11 +2224,16 @@ struct stControlSettings const ctrlSettings[] = {
 **    ssh-command      Command used to talk to a remote machine with
 **                     the "ssh://" protocol.
 **
-**    tcl              If enabled, Tcl integration commands will be added to
-**                     the TH1 interpreter, allowing Tcl expressions and
-**                     scripts to be evaluated from TH1.  Additionally, the
-**                     Tcl interpreter will be able to evaluate TH1 expressions
-**                     and scripts.  Default: off.
+**    tcl              If enabled (and Fossil was compiled with Tcl support),
+**                     Tcl integration commands will be added to the TH1
+**                     interpreter, allowing arbitrary Tcl expressions and
+**                     scripts to be evaluated from TH1.  Additionally, the Tcl
+**                     interpreter will be able to evaluate arbitrary TH1
+**                     expressions and scripts. Default: off.
+**
+**    tcl-setup        This is the setup script to be evaluated after creating
+**                     and initializing the Tcl interpreter.  By default, this
+**                     is empty and no extra setup is performed.
 **
 **    web-browser      A shell command used to launch your preferred
 **                     web browser when given a URL as an argument.
@@ -2042,7 +2243,7 @@ struct stControlSettings const ctrlSettings[] = {
 ** Options:
 **   --global   set or unset the given property globally instead of
 **              setting or unsetting it for the open repository only.
-** 
+**
 ** See also: configuration
 */
 void setting_cmd(void){
@@ -2130,7 +2331,7 @@ char *db_timespan_name(double rSpan){
 void test_timespan_cmd(void){
   double rDiff;
   if( g.argc!=3 ) usage("TIMESTAMP");
-  sqlite3_open(":memory:", &g.db);  
+  sqlite3_open(":memory:", &g.db);
   rDiff = db_double(0.0, "SELECT julianday('now') - julianday(%Q)", g.argv[2]);
   fossil_print("Time differences: %s\n", db_timespan_name(rDiff));
   sqlite3_close(g.db);
