@@ -27,17 +27,28 @@
 /*
 ** Allowed flag parameters to the text_diff() and html_sbsdiff() funtions:
 */
-#define DIFF_CONTEXT_MASK  0x0000ffff  /* Lines of context.  Default if 0 */
-#define DIFF_WIDTH_MASK    0x00ff0000  /* side-by-side column width */
-#define DIFF_IGNORE_EOLWS  0x01000000  /* Ignore end-of-line whitespace */
-#define DIFF_SIDEBYSIDE    0x02000000  /* Generate a side-by-side diff */
-#define DIFF_NEWFILE       0x04000000  /* Missing files are as empty files */
-#define DIFF_BRIEF         0x08000000  /* Show filenames only */
-#define DIFF_INLINE        0x00000000  /* Inline (not side-by-side) diff */
-#define DIFF_HTML          0x10000000  /* Render for HTML */
-#define DIFF_LINENO        0x20000000  /* Show line numbers in context diff */
-#define DIFF_NOOPT         0x40000000  /* Suppress optimizations for debug */
-#define DIFF_INVERT        0x80000000  /* Invert the diff for debug */
+#define DIFF_CONTEXT_MASK ((u64)0x0000ffff) /* Lines of context. Default if 0 */
+#define DIFF_WIDTH_MASK   ((u64)0x00ff0000) /* side-by-side column width */
+#define DIFF_IGNORE_EOLWS ((u64)0x01000000) /* Ignore end-of-line whitespace */
+#define DIFF_SIDEBYSIDE   ((u64)0x02000000) /* Generate a side-by-side diff */
+#define DIFF_NEWFILE      ((u64)0x04000000) /* Missing shown as empty files */
+#define DIFF_BRIEF        ((u64)0x08000000) /* Show filenames only */
+#define DIFF_INLINE       ((u64)0x00000000) /* Inline (not side-by-side) diff */
+#define DIFF_HTML         ((u64)0x10000000) /* Render for HTML */
+#define DIFF_LINENO       ((u64)0x20000000) /* Show line numbers */
+#define DIFF_WS_WARNING   ((u64)0x40000000) /* Warn about whitespace */
+#define DIFF_NOOPT        (((u64)0x01)<<32) /* Suppress optimizations (debug) */
+#define DIFF_INVERT       (((u64)0x02)<<32) /* Invert the diff (debug) */
+
+/*
+** These error messages are shared in multiple locations.  They are defined
+** here for consistency.
+*/
+#define DIFF_CANNOT_COMPUTE_BINARY \
+    "cannot compute difference between binary files\n"
+
+#define DIFF_CANNOT_COMPUTE_SYMLINK \
+    "cannot compute difference between symlink and regular file\n"
 
 #endif /* INTERFACE */
 
@@ -159,6 +170,34 @@ static DLine *break_into_lines(const char *z, int n, int *pnLine, int ignoreWS){
   return a;
 }
 
+/*
+** Returns non-zero if the specified content appears to be binary or
+** contains a line that is too long.
+*/
+int looks_like_binary(Blob *pContent){
+  const char *z = blob_str(pContent);
+  int n = blob_size(pContent);
+  int i, j;
+
+  /* Count the number of lines.  Allocate space to hold
+  ** the returned array.
+  */
+  for(i=j=0; i<n; i++, j++){
+    int c = z[i];
+    if( c==0 ) return 1;  /* \000 byte in a file -> binary */
+    if( c=='\n' && z[i+1]!=0 ){
+      if( j>LENGTH_MASK ){
+        return 1;   /* Very long line -> binary */
+      }
+      j = 0;
+    }
+  }
+  if( j>LENGTH_MASK ){
+    return 1;  /* Very long line -> binary */
+  }
+  return 0;   /* No problems seen -> not binary */
+}
+
 /*
 ** Return true if two DLine elements are identical.
 */
@@ -175,6 +214,7 @@ static void appendDiffLine(
   DLine *pLine,       /* The line to be output */
   int html            /* True if generating HTML.  False for plain text */
 ){
+  int i;
   blob_append(pOut, &cPrefix, 1);
   if( html ){
     char *zHtml;
@@ -184,6 +224,10 @@ static void appendDiffLine(
       blob_append(pOut, "<span class=\"diffrm\">", -1);
     }
     zHtml = htmlize(pLine->z, (pLine->h & LENGTH_MASK));
+    for(i=0; i<strlen(zHtml); i++){
+      char c = zHtml[i];
+      if( c=='\t' || c=='\r' || c=='\f' ) zHtml[i] = ' ';
+    }
     blob_append(pOut, zHtml, -1);
     fossil_free(zHtml);
     if( cPrefix!=' ' ){
@@ -377,9 +421,12 @@ struct SbsLine {
 #define SBS_PAD          0x0002   /* Pad output to width spaces */
 
 /*
-** Write up to width characters of pLine into z[].  Translate tabs into
+** Write up to width characters of pLine into p->zLine[].  Translate tabs into
 ** spaces.  Add a newline if SBS_NEWLINE is set.  Translate HTML characters
 ** if SBS_HTML is set.  Pad the rendering out width bytes if SBS_PAD is set.
+**
+** This comment contains multibyte unicode characters (ü, Æ, ð) in order
+** to test the ability of the diff code to handle such characters.
 */
 static void sbsWriteText(SbsLine *p, DLine *pLine, unsigned flags){
   int n = pLine->h & LENGTH_MASK;
@@ -427,8 +474,12 @@ static void sbsWriteText(SbsLine *p, DLine *pLine, unsigned flags){
     }else if( c=='>' && p->escHtml ){
       memcpy(&z[j], "&gt;", 4);
       j += 4;
+    }else if( c=='"' && p->escHtml ){
+      memcpy(&z[j], "&quot;", 6);
+      j += 6;
     }else{
       z[j++] = c;
+      if( (c&0xc0)==0x80 ) k--;
     }
   }
   if( needEndSpan ){
@@ -445,7 +496,7 @@ static void sbsWriteText(SbsLine *p, DLine *pLine, unsigned flags){
 }
 
 /*
-** Append a string to an SbSLine with coding, interpretation, or padding.
+** Append a string to an SbSLine without coding, interpretation, or padding.
 */
 static void sbsWrite(SbsLine *p, const char *zIn, int nIn){
   memcpy(p->zLine+p->n, zIn, nIn);
@@ -884,11 +935,12 @@ static void sbsDiff(
   SbsLine s;    /* Output line buffer */
 
   memset(&s, 0, sizeof(s));
-  s.zLine = fossil_malloc( 10*width + 200 );
+  s.zLine = fossil_malloc( 15*width + 200 );
   if( s.zLine==0 ) return;
   s.width = width;
   s.escHtml = escHtml;
   s.iStart = -1;
+  s.iStart2 = 0;
   s.iEnd = -1;
   A = p->aFrom;
   B = p->aTo;
@@ -971,7 +1023,11 @@ static void sbsDiff(
           s.zStart = "<span class=\"diffrm\">";
           s.iEnd = s.width;
           sbsWriteText(&s, &A[a], SBS_PAD);
-          sbsWrite(&s, " <\n", 3);
+          if( escHtml ){
+            sbsWrite(&s, " &lt;\n", 6);
+          }else{
+            sbsWrite(&s, " <\n", 3);
+          }
           blob_append(pOut, s.zLine, s.n);
           assert( ma>0 );
           ma--;
@@ -988,7 +1044,11 @@ static void sbsDiff(
         }else{
           s.n = 0;
           sbsWriteSpace(&s, width + 7);
-          sbsWrite(&s, " > ", 3);
+          if( escHtml ){
+            sbsWrite(&s, " &gt; ", 6);
+          }else{
+            sbsWrite(&s, " > ", 3);
+          }
           sbsWriteLineno(&s, b);
           s.iStart = 0;
           s.zStart = "<span class=\"diffadd\">";
@@ -1164,7 +1224,7 @@ static void longestCommonSequence(
       iSYb = iSY;
       iEXb = iEX;
       iEYb = iEY;
-    }else{
+    }else if( iEX>iEXp ){
       iSXp = iSX;
       iSYp = iSY;
       iEXp = iEX;
@@ -1424,7 +1484,7 @@ static void diff_optimize(DContext *p){
 ** Extract the number of lines of context from diffFlags.  Supply an
 ** appropriate default if no context width is specified.
 */
-int diff_context_lines(int diffFlags){
+int diff_context_lines(u64 diffFlags){
   int n = diffFlags & DIFF_CONTEXT_MASK;
   if( n==0 ) n = 5;
   return n;
@@ -1434,7 +1494,7 @@ int diff_context_lines(int diffFlags){
 ** Extract the width of columns for side-by-side diff.  Supply an
 ** appropriate default if no width is given.
 */
-int diff_width(int diffFlags){
+int diff_width(u64 diffFlags){
   int w = (diffFlags & DIFF_WIDTH_MASK)/(DIFF_CONTEXT_MASK+1);
   if( w==0 ) w = 80;
   return w;
@@ -1458,7 +1518,7 @@ int *text_diff(
   Blob *pA_Blob,   /* FROM file */
   Blob *pB_Blob,   /* TO file */
   Blob *pOut,      /* Write diff here if not NULL */
-  int diffFlags    /* DIFF_* flags defined above */
+  u64 diffFlags    /* DIFF_* flags defined above */
 ){
   int ignoreEolWs; /* Ignore whitespace at the end of lines */
   int nContext;    /* Amount of context to display */	
@@ -1479,10 +1539,10 @@ int *text_diff(
   c.aTo = break_into_lines(blob_str(pB_Blob), blob_size(pB_Blob),
                            &c.nTo, ignoreEolWs);
   if( c.aFrom==0 || c.aTo==0 ){
-    free(c.aFrom);
-    free(c.aTo);
+    fossil_free(c.aFrom);
+    fossil_free(c.aTo);
     if( pOut ){
-      blob_appendf(pOut, "cannot compute difference between binary files\n");
+      blob_appendf(pOut, DIFF_CANNOT_COMPUTE_BINARY);
     }
     return 0;
   }
@@ -1501,9 +1561,9 @@ int *text_diff(
       int showLn = (diffFlags & DIFF_LINENO)!=0;
       contextDiff(&c, pOut, nContext, showLn, escHtml);
     }
-    free(c.aFrom);
-    free(c.aTo);
-    free(c.aEdit);
+    fossil_free(c.aFrom);
+    fossil_free(c.aTo);
+    fossil_free(c.aEdit);
     return 0;
   }else{
     /* If a context diff is not requested, then return the
@@ -1526,13 +1586,15 @@ int *text_diff(
 **   --linenum|-n           Show line numbers      DIFF_LINENO
 **   --noopt                Disable optimization   DIFF_NOOPT
 **   --side-by-side|-y      Side-by-side diff.     DIFF_SIDEBYSIDE
+**   --unified              Unified diff.          ~DIFF_SIDEBYSIDE
 **   --width|-W N           N character lines.     DIFF_WIDTH_MASK
 */
 int diff_options(void){
-  int diffFlags = 0;
+  u64 diffFlags = 0;
   const char *z;
   int f;
   if( find_option("side-by-side","y",0)!=0 ) diffFlags |= DIFF_SIDEBYSIDE;
+  if( find_option("unified",0,0)!=0 ) diffFlags &= ~DIFF_SIDEBYSIDE;
   if( (z = find_option("context","c",1))!=0 && (f = atoi(z))>0 ){
     if( f > DIFF_CONTEXT_MASK ) f = DIFF_CONTEXT_MASK;
     diffFlags |= f;
@@ -1558,7 +1620,7 @@ void test_rawdiff_cmd(void){
   int r;
   int i;
   int *R;
-  int diffFlags = diff_options();
+  u64 diffFlags = diff_options();
   if( g.argc<4 ) usage("FILE1 FILE2 ...");
   blob_read_from_file(&a, g.argv[2]);
   for(i=3; i<g.argc; i++){
@@ -1580,7 +1642,7 @@ void test_rawdiff_cmd(void){
 */
 void test_udiff_cmd(void){
   Blob a, b, out;
-  int diffFlag = diff_options();
+  u64 diffFlag = diff_options();
 
   if( g.argc!=4 ) usage("FILE1 FILE2");
   blob_read_from_file(&a, g.argv[2]);
@@ -1680,7 +1742,7 @@ static int annotation_step(Annotator *p, Blob *pParent, char *zPName){
   }
 
   /* Clear out the diff results */
-  free(p->c.aEdit);
+  fossil_free(p->c.aEdit);
   p->c.aEdit = 0;
   p->c.nEdit = 0;
   p->c.nEditAlloc = 0;
@@ -1779,8 +1841,8 @@ static void annotate_file(
     const char *zUser = db_column_text(&q, 3);
     if( webLabel ){
       zLabel = mprintf(
-          "<a href='%s/info/%s' target='infowindow'>%.10s</a> %s %13.13s", 
-          g.zTop, zUuid, zUuid, zDate, zUser
+          "<a href='%R/info/%s' target='infowindow'>%.10s</a> %s %13.13s", 
+          zUuid, zUuid, zDate, zUser
       );
     }else{
       zLabel = mprintf("%.10s %s %13.13s", zUuid, zDate, zUser);
@@ -1822,7 +1884,7 @@ void annotation_page(void){
   }
   style_header("File Annotation");
   if( P("filevers") ) annFlags |= ANN_FILE_VERS;
-  annotate_file(&ann, fnid, mid, g.perm.History, iLimit, annFlags);
+  annotate_file(&ann, fnid, mid, g.perm.Hyperlink, iLimit, annFlags);
   if( P("log") ){
     int i;
     @ <h2>Versions analyzed:</h2>
