@@ -30,6 +30,15 @@
 #include <errno.h>
 #include "file.h"
 
+/*
+** On Windows, include the Platform SDK header file.
+*/
+#ifdef _WIN32
+# include <direct.h>
+# include <windows.h>
+# include <sys/utime.h>
+#endif
+
 /*
 ** The file status information from the most recent stat() call.
 **
@@ -37,12 +46,13 @@
 ** larger than 2GB.
 */
 #if defined(_WIN32) && (defined(__MSVCRT__) || defined(_MSC_VER))
+# undef stat
 # define stat _stati64
 #endif
 /*
 ** On Windows S_ISLNK always returns FALSE.
 */
-#if defined(_WIN32)
+#if !defined(S_ISLNK)
 # define S_ISLNK(x) (0)
 #endif
 static int fileStatValid = 0;
@@ -62,8 +72,8 @@ static int fossil_stat(const char *zFilename, struct stat *buf, int isWd){
   }
 #else
   int rc = 0;
-  char *zMbcs = fossil_utf8_to_mbcs(zFilename);
-  rc = stat(zMbcs, buf);
+  wchar_t *zMbcs = fossil_utf8_to_unicode(zFilename);
+  rc = _wstati64(zMbcs, buf);
   fossil_mbcs_free(zMbcs);
   return rc;
 #endif
@@ -107,7 +117,7 @@ i64 file_size(const char *zFilename){
 ** Same as file_size(), but takes into account symlinks.
 */
 i64 file_wd_size(const char *zFilename){
-  return getStat(zFilename, 1) ? -1 : fileStat.st_size;  
+  return getStat(zFilename, 1) ? -1 : fileStat.st_size;
 }
 
 /*
@@ -127,7 +137,7 @@ i64 file_wd_mtime(const char *zFilename){
 }
 
 /*
-** Return TRUE if the named file is an ordinary file or symlink 
+** Return TRUE if the named file is an ordinary file or symlink
 ** and symlinks are allowed.
 ** Return false for directories, devices, fifos, etc.
 */
@@ -170,7 +180,7 @@ void symlink_create(const char *zTargetFile, const char *zLinkFile){
       zName = zBuf;
       memcpy(zName, zLinkFile, nName+1);
     }
-    nName = file_simplify_name(zName, nName);
+    nName = file_simplify_name(zName, nName, 0);
     for(i=1; i<nName; i++){
       if( zName[i]=='/' ){
         zName[i] = 0;
@@ -184,10 +194,10 @@ void symlink_create(const char *zTargetFile, const char *zLinkFile){
     if( zName!=zBuf ) free(zName);
 
     if( symlink(zTargetFile, zName)!=0 ){
-      fossil_fatal_recursive("unable to create symlink \"%s\"", zName);      
+      fossil_fatal_recursive("unable to create symlink \"%s\"", zName);
     }
   }else
-#endif 
+#endif
   {
     Blob content;
     blob_set(&content, zTargetFile);
@@ -215,7 +225,7 @@ void symlink_copy(const char *zFrom, const char *zTo){
 int file_wd_perm(const char *zFilename){
   if( getStat(zFilename, 1) ) return PERM_REG;
 #if defined(_WIN32)
-#  if defined(__DMC__) || defined(_MSC_VER)
+#  ifndef S_IXUSR
 #    define S_IXUSR  _S_IEXEC
 #  endif
   if( S_ISREG(fileStat.st_mode) && ((S_IXUSR)&fileStat.st_mode)!=0 )
@@ -223,7 +233,7 @@ int file_wd_perm(const char *zFilename){
   else
     return PERM_REG;
 #else
-  if( S_ISREG(fileStat.st_mode) && 
+  if( S_ISREG(fileStat.st_mode) &&
       ((S_IXUSR|S_IXGRP|S_IXOTH)&fileStat.st_mode)!=0 )
     return PERM_EXE;
   else if( g.allowSymlinks && S_ISLNK(fileStat.st_mode) )
@@ -261,7 +271,7 @@ int file_isdir(const char *zFilename){
 
   if( zFilename ){
     char *zFN = mprintf("%s", zFilename);
-    file_simplify_name(zFN, -1);
+    file_simplify_name(zFN, -1, 0);
     rc = getStat(zFN, 0);
     free(zFN);
   }else{
@@ -278,7 +288,7 @@ int file_wd_isdir(const char *zFilename){
 
   if( zFilename ){
     char *zFN = mprintf("%s", zFilename);
-    file_simplify_name(zFN, -1);
+    file_simplify_name(zFN, -1, 0);
     rc = getStat(zFN, 1);
     free(zFN);
   }else{
@@ -292,9 +302,13 @@ int file_wd_isdir(const char *zFilename){
 ** Wrapper around the access() system call.
 */
 int file_access(const char *zFilename, int flags){
-  char *zMbcs = fossil_utf8_to_mbcs(zFilename);
-  int rc = access(zMbcs, flags);
+#ifdef _WIN32
+  wchar_t *zMbcs = fossil_utf8_to_unicode(zFilename);
+  int rc = _waccess(zMbcs, flags);
   fossil_mbcs_free(zMbcs);
+#else
+  int rc = access(zFilename, flags);
+#endif
   return rc;
 }
 
@@ -316,7 +330,7 @@ char *file_newname(const char *zBase, const char *zSuffix, int relFlag){
   }
   if( relFlag ){
     Blob x;
-    file_relative_name(z, &x);
+    file_relative_name(z, &x, 0);
     fossil_free(z);
     z = blob_str(&x);
   }
@@ -379,18 +393,65 @@ int file_wd_setexe(const char *zFilename, int onoff){
   return rc;
 }
 
+/*
+** Set the mtime for a file.
+*/
+void file_set_mtime(const char *zFilename, i64 newMTime){
+#if !defined(_WIN32)
+  struct timeval tv[2];
+  memset(tv, 0, sizeof(tv[0])*2);
+  tv[0].tv_sec = newMTime;
+  tv[1].tv_sec = newMTime;
+  utimes(zFilename, tv);
+#else
+  struct _utimbuf tb;
+  wchar_t *zMbcs = fossil_utf8_to_unicode(zFilename);
+  tb.actime = newMTime;
+  tb.modtime = newMTime;
+  _wutime(zMbcs, &tb);
+  fossil_mbcs_free(zMbcs);
+#endif
+}
+
+/*
+** COMMAND: test-set-mtime
+**
+** Usage: %fossil test-set-mtime FILENAME DATE/TIME
+**
+** Sets the mtime of the named file to the date/time shown.
+*/
+void test_set_mtime(void){
+  const char *zFile;
+  char *zDate;
+  i64 iMTime;
+  if( g.argc!=4 ){
+    usage("test-set-mtime FILENAME DATE/TIME");
+  }
+  db_open_or_attach(":memory:", "mem");
+  iMTime = db_int64(0, "SELECT strftime('%%s',%Q)", g.argv[3]);
+  zFile = g.argv[2];
+  file_set_mtime(zFile, iMTime);
+  iMTime = file_wd_mtime(zFile);
+  zDate = db_text(0, "SELECT datetime(%lld, 'unixepoch')", iMTime);
+  fossil_print("Set mtime of \"%s\" to %s (%lld)\n", zFile, zDate, iMTime);
+}
+
 /*
 ** Delete a file.
 */
 void file_delete(const char *zFilename){
-  char *z = fossil_utf8_to_mbcs(zFilename);
-  unlink(z);
+#ifdef _WIN32
+  wchar_t *z = fossil_utf8_to_unicode(zFilename);
+  _wunlink(z);
   fossil_mbcs_free(z);
+#else
+  unlink(zFilename);
+#endif
 }
 
 /*
 ** Create the directory named in the argument, if it does not already
-** exist.  If forceFlag is 1, delete any prior non-directory object 
+** exist.  If forceFlag is 1, delete any prior non-directory object
 ** with the same name.
 **
 ** Return the number of errors.
@@ -404,8 +465,8 @@ int file_mkdir(const char *zName, int forceFlag){
   if( rc!=1 ){
 #if defined(_WIN32)
     int rc;
-    char *zMbcs = fossil_utf8_to_mbcs(zName);
-    rc = mkdir(zMbcs);
+    wchar_t *zMbcs = fossil_utf8_to_unicode(zName);
+    rc = _wmkdir(zMbcs);
     fossil_mbcs_free(zMbcs);
     return rc;
 #else
@@ -475,8 +536,10 @@ static int backup_dir(const char *z, int *pJ){
 **  * removing /A/../
 **
 ** Changes are made in-place.  Return the new name length.
+** If the slash parameter is non-zero, the trailing slash, if any,
+** is retained.
 */
-int file_simplify_name(char *z, int n){
+int file_simplify_name(char *z, int n, int slash){
   int i, j;
   if( n<0 ) n = strlen(z);
 
@@ -488,7 +551,9 @@ int file_simplify_name(char *z, int n){
 #endif
 
   /* Removing trailing "/" characters */
-  while( n>1 && z[n-1]=='/' ){ n--; }
+  if ( !slash ){
+    while( n>1 && z[n-1]=='/' ){ n--; }
+  }
 
   /* Remove duplicate '/' characters.  Except, two // at the beginning
   ** of a pathname is allowed since this is important on windows. */
@@ -542,7 +607,7 @@ void cmd_test_simplify_name(void){
   for(i=2; i<g.argc; i++){
     z = mprintf("%s", g.argv[i]);
     fossil_print("[%s] -> ", z);
-    file_simplify_name(z, -1);
+    file_simplify_name(z, -1, 0);
     fossil_print("[%s]\n", z);
     fossil_free(z);
   }
@@ -551,7 +616,7 @@ void cmd_test_simplify_name(void){
 /*
 ** Get the current working directory.
 **
-** On windows, the name is converted from MBCS to UTF8 and all '\\'
+** On windows, the name is converted from unicode to UTF8 and all '\\'
 ** characters are converted to '/'.  No conversions are needed on
 ** unix.
 */
@@ -560,11 +625,11 @@ void file_getcwd(char *zBuf, int nBuf){
   char *zPwdUtf8;
   int nPwd;
   int i;
-  char zPwd[2000];
-  if( getcwd(zPwd, sizeof(zPwd)-1)==0 ){
+  wchar_t zPwd[2000];
+  if( _wgetcwd(zPwd, sizeof(zPwd)/sizeof(zPwd[0])-1)==0 ){
     fossil_fatal("cannot find the current working directory.");
   }
-  zPwdUtf8 = fossil_mbcs_to_utf8(zPwd);
+  zPwdUtf8 = fossil_unicode_to_utf8(zPwd);
   nPwd = strlen(zPwdUtf8);
   if( nPwd > nBuf-1 ){
     fossil_fatal("pwd too big: max %d\n", nBuf-1);
@@ -608,18 +673,41 @@ int file_is_absolute_path(const char *zPath){
 ** Remove redundant / characters
 ** Remove all /./ path elements.
 ** Convert /A/../ to just /
+** If the slash parameter is non-zero, the trailing slash, if any,
+** is retained.
 */
-void file_canonical_name(const char *zOrigName, Blob *pOut){
+void file_canonical_name(const char *zOrigName, Blob *pOut, int slash){
   if( file_is_absolute_path(zOrigName) ){
+#if defined(_WIN32)
+    char *zOut;
+#endif
     blob_set(pOut, zOrigName);
     blob_materialize(pOut);
+#if defined(_WIN32)
+    /*
+    ** On Windows, normalize the drive letter to upper case.
+    */
+    zOut = blob_str(pOut);
+    if( fossil_isalpha(zOut[0]) && zOut[1]==':' ){
+      zOut[0] = fossil_toupper(zOut[0]);
+    }
+#endif
   }else{
     char zPwd[2000];
     file_getcwd(zPwd, sizeof(zPwd)-strlen(zOrigName));
+#if defined(_WIN32)
+    /*
+    ** On Windows, normalize the drive letter to upper case.
+    */
+    if( fossil_isalpha(zPwd[0]) && zPwd[1]==':' ){
+      zPwd[0] = fossil_toupper(zPwd[0]);
+    }
+#endif
     blob_zero(pOut);
     blob_appendf(pOut, "%//%/", zPwd, zOrigName);
   }
-  blob_resize(pOut, file_simplify_name(blob_buffer(pOut), blob_size(pOut)));
+  blob_resize(pOut, file_simplify_name(blob_buffer(pOut),
+                                       blob_size(pOut), slash));
 }
 
 /*
@@ -636,7 +724,7 @@ void cmd_test_canonical_name(void){
   for(i=2; i<g.argc; i++){
     char zBuf[100];
     const char *zName = g.argv[i];
-    file_canonical_name(zName, &x);
+    file_canonical_name(zName, &x, 0);
     fossil_print("[%s] -> [%s]\n", zName, blob_buffer(&x));
     blob_reset(&x);
     sqlite3_snprintf(sizeof(zBuf), zBuf, "%lld", file_wd_size(zName));
@@ -677,7 +765,7 @@ int file_is_canonical(const char *z){
   return 1;
 }
 
-/* 
+/*
 ** Return a pointer to the first character in a pathname past the
 ** drive letter.  This routine is a no-op on unix.
 */
@@ -690,12 +778,14 @@ char *file_without_drive_letter(char *zIn){
 
 /*
 ** Compute a pathname for a file or directory that is relative
-** to the current directory.
+** to the current directory.  If the slash parameter is non-zero,
+** the trailing slash, if any, is retained.
 */
-void file_relative_name(const char *zOrigName, Blob *pOut){
+void file_relative_name(const char *zOrigName, Blob *pOut, int slash){
   char *zPath;
   blob_set(pOut, zOrigName);
-  blob_resize(pOut, file_simplify_name(blob_buffer(pOut), blob_size(pOut))); 
+  blob_resize(pOut, file_simplify_name(blob_buffer(pOut),
+                                       blob_size(pOut), slash));
   zPath = file_without_drive_letter(blob_buffer(pOut));
   if( zPath[0]=='/' ){
     int i, j;
@@ -755,7 +845,7 @@ void cmd_test_relative_name(void){
   Blob x;
   blob_zero(&x);
   for(i=2; i<g.argc; i++){
-    file_relative_name(g.argv[i], &x);
+    file_relative_name(g.argv[i], &x, 0);
     fossil_print("%s\n", blob_buffer(&x));
     blob_reset(&x);
   }
@@ -770,33 +860,42 @@ void cmd_test_relative_name(void){
 ** The root of the tree is defined by the g.zLocalRoot variable.
 */
 int file_tree_name(const char *zOrigName, Blob *pOut, int errFatal){
-  int n;
+  Blob localRoot;
+  int nLocalRoot;
+  char *zLocalRoot;
   Blob full;
   int nFull;
   char *zFull;
 
   blob_zero(pOut);
   db_must_be_within_tree();
-  file_canonical_name(zOrigName, &full);
-  n = strlen(g.zLocalRoot);
-  assert( n>0 && g.zLocalRoot[n-1]=='/' );
+  file_canonical_name(g.zLocalRoot, &localRoot, 1);
+  nLocalRoot = blob_size(&localRoot);
+  zLocalRoot = blob_buffer(&localRoot);
+  assert( nLocalRoot>0 && zLocalRoot[nLocalRoot-1]=='/' );
+  file_canonical_name(zOrigName, &full, 0);
   nFull = blob_size(&full);
   zFull = blob_buffer(&full);
 
   /* Special case.  zOrigName refers to g.zLocalRoot directory. */
-  if( nFull==n-1 && memcmp(g.zLocalRoot, zFull, nFull)==0 ){
+  if( nFull==nLocalRoot-1 && memcmp(zLocalRoot, zFull, nFull)==0 ){
     blob_append(pOut, ".", 1);
+    blob_reset(&localRoot);
+    blob_reset(&full);
     return 1;
   }
 
-  if( nFull<=n || memcmp(g.zLocalRoot, zFull, n) ){
+  if( nFull<=nLocalRoot || memcmp(zLocalRoot, zFull, nLocalRoot) ){
+    blob_reset(&localRoot);
     blob_reset(&full);
     if( errFatal ){
       fossil_fatal("file outside of checkout tree: %s", zOrigName);
     }
     return 0;
   }
-  blob_append(pOut, &zFull[n], nFull-n);
+  blob_append(pOut, &zFull[nLocalRoot], nFull-nLocalRoot);
+  blob_reset(&localRoot);
+  blob_reset(&full);
   return 1;
 }
 
@@ -863,10 +962,16 @@ void file_parse_uri(
 */
 void file_tempname(int nBuf, char *zBuf){
   static const char *azDirs[] = {
+#if defined(_WIN32)
+     0, /* GetTempPath */
+     0, /* TEMP */
+     0, /* TMP */
+#else
      "/var/tmp",
      "/usr/tmp",
      "/tmp",
      "/temp",
+#endif
      ".",
   };
   static const unsigned char zChars[] =
@@ -876,14 +981,27 @@ void file_tempname(int nBuf, char *zBuf){
   unsigned int i, j;
   const char *zDir = ".";
   int cnt = 0;
-  
+
+#if defined(_WIN32)
+  wchar_t zTmpPath[MAX_PATH];
+
+  if( GetTempPathW(MAX_PATH, zTmpPath) ){
+    azDirs[0] = fossil_unicode_to_utf8(zTmpPath);
+  }
+
+  azDirs[1] = fossil_getenv("TEMP");
+  azDirs[2] = fossil_getenv("TMP");
+#endif
+
+
   for(i=0; i<sizeof(azDirs)/sizeof(azDirs[0]); i++){
+    if( azDirs[i]==0 ) continue;
     if( !file_isdir(azDirs[i]) ) continue;
     zDir = azDirs[i];
     break;
   }
 
-  /* Check that the output buffer is large enough for the temporary file 
+  /* Check that the output buffer is large enough for the temporary file
   ** name. If it is not, return SQLITE_ERROR.
   */
   if( (strlen(zDir) + 17) >= (size_t)nBuf ){
@@ -900,6 +1018,11 @@ void file_tempname(int nBuf, char *zBuf){
     }
     zBuf[j] = 0;
   }while( file_size(zBuf)>=0 );
+
+#if defined(_WIN32)
+  fossil_mbcs_free((char *)azDirs[1]);
+  fossil_mbcs_free((char *)azDirs[2]);
+#endif
 }
 
 
@@ -926,18 +1049,32 @@ int file_is_the_same(Blob *pContent, const char *zName){
   return rc==0;
 }
 
+/*
+** Portable unicode implementation of opendir()
+*/
+#if INTERFACE
+
+#include <dirent.h>
+#if defined(_WIN32)
+# define DIR _WDIR
+# define dirent _wdirent
+# define opendir _wopendir
+# define readdir _wreaddir
+# define closedir _wclosedir
+#endif /* _WIN32 */
+
+#endif /* INTERFACE */
+
+
 
 /**************************************************************************
 ** The following routines translate between MBCS and UTF8 on windows.
 ** Since everything is always UTF8 on unix, these routines are no-ops
 ** there.
 */
-#ifdef _WIN32
-# include <windows.h>
-#endif
 
 /*
-** Translate MBCS to UTF8.  Return a pointer to the translated text.  
+** Translate MBCS to UTF8.  Return a pointer to the translated text.
 ** Call fossil_mbcs_free() to deallocate any memory used to store the
 ** returned pointer when done.
 */
@@ -947,7 +1084,26 @@ char *fossil_mbcs_to_utf8(const char *zMbcs){
   return sqlite3_win32_mbcs_to_utf8(zMbcs);
 #else
   return (char*)zMbcs;  /* No-op on unix */
-#endif  
+#endif
+}
+
+/*
+** Translate Unicode to UTF8.  Return a pointer to the translated text.
+** Call fossil_mbcs_free() to deallocate any memory used to store the
+** returned pointer when done.
+*/
+char *fossil_unicode_to_utf8(const void *zUnicode){
+#ifdef _WIN32
+  int nByte = WideCharToMultiByte(CP_UTF8, 0, zUnicode, -1, 0, 0, 0, 0);
+  char *zUtf = sqlite3_malloc( nByte );
+  if( zUtf==0 ){
+    return 0;
+  }
+  WideCharToMultiByte(CP_UTF8, 0, zUnicode, -1, zUtf, nByte, 0, 0);
+  return zUtf;
+#else
+  return (char *)zUnicode;  /* No-op on unix */
+#endif
 }
 
 /*
@@ -961,80 +1117,107 @@ char *fossil_utf8_to_mbcs(const char *zUtf8){
   return sqlite3_win32_utf8_to_mbcs(zUtf8);
 #else
   return (char*)zUtf8;  /* No-op on unix */
-#endif  
+#endif
+}
+
+/*
+** Translate UTF8 to unicode for use in system calls.  Return a pointer to the
+** translated text..  Call fossil_mbcs_free() to deallocate any memory
+** used to store the returned pointer when done.
+*/
+void *fossil_utf8_to_unicode(const char *zUtf8){
+#ifdef _WIN32
+  int nByte = MultiByteToWideChar(CP_UTF8, 0, zUtf8, -1, 0, 0);
+  wchar_t *zUnicode = sqlite3_malloc( nByte * 2 );
+  if( zUnicode==0 ){
+    return 0;
+  }
+  MultiByteToWideChar(CP_UTF8, 0, zUtf8, -1, zUnicode, nByte);
+  return zUnicode;
+#else
+  return (void *)zUtf8;  /* No-op on unix */
+#endif
 }
 
 /*
 ** Return the value of an environment variable as UTF8.
 */
 char *fossil_getenv(const char *zName){
-  char *zValue = getenv(zName);
 #ifdef _WIN32
-  if( zValue ) zValue = fossil_mbcs_to_utf8(zValue);
+  wchar_t *uName = fossil_utf8_to_unicode(zName);
+  void *zValue = _wgetenv(uName);
+  fossil_mbcs_free(uName);
+  if( zValue ) zValue = fossil_unicode_to_utf8(zValue);
+#else
+  char *zValue = getenv(zName);
 #endif
   return zValue;
 }
 
 /*
-** Translate UTF8 to MBCS for display on the console.  Return a pointer to the
-** translated text..  Call fossil_mbcs_free() to deallocate any memory
-** used to store the returned pointer when done.
+** Display UTF8 on the console.  Return the number of
+** Characters written. If stdout or stderr is redirected
+** to a file, -1 is returned and nothing is written
+** to the console.
 */
-char *fossil_utf8_to_console(const char *zUtf8){
+int fossil_utf8_to_console(const char *zUtf8, int nByte, int toStdErr){
 #ifdef _WIN32
-  int nChar, nByte;
-  WCHAR *zUnicode;   /* Unicode version of zUtf8 */
-  char *zConsole;    /* Console version of zUtf8 */
-  int codepage;      /* Console code page */
+  int nChar;
+  wchar_t *zUnicode; /* Unicode version of zUtf8 */
+  DWORD dummy;
 
-  nChar = MultiByteToWideChar(CP_UTF8, 0, zUtf8, -1, NULL, 0);
-  zUnicode = malloc( nChar*sizeof(zUnicode[0]) );
+  static int istty[2] = { -1, -1 };
+  if( istty[toStdErr] == -1 ){
+    istty[toStdErr] = _isatty(toStdErr + 1) != 0;
+  }
+  if( !istty[toStdErr] ){
+    /* stdout/stderr is not a console. */
+    return -1;
+  }
+
+  nChar = MultiByteToWideChar(CP_UTF8, 0, zUtf8, nByte, NULL, 0);
+  zUnicode = malloc( (nChar + 1) *sizeof(zUnicode[0]) );
   if( zUnicode==0 ){
     return 0;
   }
-  nChar = MultiByteToWideChar(CP_UTF8, 0, zUtf8, -1, zUnicode, nChar);
+  nChar = MultiByteToWideChar(CP_UTF8, 0, zUtf8, nByte, zUnicode, nChar);
   if( nChar==0 ){
     free(zUnicode);
     return 0;
   }
-  codepage = GetConsoleCP();
-  nByte = WideCharToMultiByte(codepage, 0, zUnicode, -1, 0, 0, 0, 0);
-  zConsole = malloc( nByte );
-  if( zConsole==0 ){
-    free(zUnicode);
-    return 0;
-  }
-  nByte = WideCharToMultiByte(codepage, 0, zUnicode, -1, zConsole, nByte, 0, 0);
-  free(zUnicode);
-  if( nByte == 0 ){
-    free(zConsole);
-    zConsole = 0;
-  }
-  return zConsole;
+  zUnicode[nChar] = '\0';
+  WriteConsoleW(GetStdHandle(STD_OUTPUT_HANDLE - toStdErr), zUnicode, nChar, &dummy, 0);
+  return nChar;
 #else
-  return (char*)zUtf8;  /* No-op on unix */
-#endif  
+  return -1;  /* No-op on unix */
+#endif
 }
 
 /*
 ** Translate MBCS to UTF8.  Return a pointer.  Call fossil_mbcs_free()
 ** to deallocate any memory used to store the returned pointer when done.
 */
-void fossil_mbcs_free(char *zOld){
+void fossil_mbcs_free(void *zOld){
 #ifdef _WIN32
   extern void sqlite3_free(void*);
   sqlite3_free(zOld);
 #else
   /* No-op on unix */
-#endif  
+#endif
 }
 
 /*
 ** Like fopen() but always takes a UTF8 argument.
 */
 FILE *fossil_fopen(const char *zName, const char *zMode){
-  char *zMbcs = fossil_utf8_to_mbcs(zName);
-  FILE *f = fopen(zMbcs, zMode);
-  fossil_mbcs_free(zMbcs);
+#ifdef _WIN32
+  wchar_t *uMode = fossil_utf8_to_unicode(zMode);
+  wchar_t *uName = fossil_utf8_to_unicode(zName);
+  FILE *f = _wfopen(uName, uMode);
+  fossil_mbcs_free(uName);
+  fossil_mbcs_free(uMode);
+#else
+  FILE *f = fopen(zName, zMode);
+#endif
   return f;
 }
