@@ -20,7 +20,6 @@
 #include "config.h"
 #include "rebuild.h"
 #include <assert.h>
-#include <dirent.h>
 #include <errno.h>
 
 /*
@@ -748,7 +747,7 @@ void test_clusters_cmd(void){
 ** %fossil scrub ?OPTIONS? ?REPOSITORY?
 **
 ** The command removes sensitive information (such as passwords) from a
-** repository so that the respository can be sent to an untrusted reader.
+** repository so that the repository can be sent to an untrusted reader.
 **
 ** By default, only passwords are removed.  However, if the --verily option
 ** is added, then private branches, concealed email addresses, IP
@@ -772,22 +771,13 @@ void scrub_cmd(void){
   int bForce = find_option("force", "f", 0)!=0;
   int privateOnly = find_option("private",0,0)!=0;
   int bNeedRebuild = 0;
-  if( g.argc!=2 && g.argc!=3 ) usage("?REPOSITORY?");
-  if( g.argc==2 ){
-    db_find_and_open_repository(OPEN_ANY_SCHEMA, 0);
-    if( g.argc!=2 ){
-      usage("?REPOSITORY-FILENAME?");
-    }
-    db_close(1);
-    db_open_repository(g.zRepositoryName);
-  }else{
-    db_open_repository(g.argv[2]);
-  }
+  db_find_and_open_repository(OPEN_ANY_SCHEMA, 2);
   if( !bForce ){
     Blob ans;
     blob_zero(&ans);
-    prompt_user("Scrubbing the repository will permanently information.\n"
-                "Changes cannot be undone.  Continue (y/N)? ", &ans);
+    prompt_user(
+         "Scrubbing the repository will permanently delete information.\n"
+         "Changes cannot be undone.  Continue (y/N)? ", &ans);
     if( blob_str(&ans)[0]!='y' ){
       fossil_exit(1);
     }
@@ -833,11 +823,11 @@ void recon_read_dir(char *zPath){
   struct dirent *pEntry;
   Blob aContent; /* content of the just read artifact */
   static int nFileRead = 0;
-  char *zMbcsPath;
+  void *zUnicodePath;
   char *zUtf8Name;
 
-  zMbcsPath = fossil_utf8_to_mbcs(zPath);
-  d = opendir(zMbcsPath);
+  zUnicodePath = fossil_utf8_to_unicode(zPath);
+  d = opendir(zUnicodePath);
   if( d ){
     while( (pEntry=readdir(d))!=0 ){
       Blob path;
@@ -846,7 +836,7 @@ void recon_read_dir(char *zPath){
       if( pEntry->d_name[0]=='.' ){
         continue;
       }
-      zUtf8Name = fossil_mbcs_to_utf8(pEntry->d_name);
+      zUtf8Name = fossil_unicode_to_utf8(pEntry->d_name);
       zSubpath = mprintf("%s/%s", zPath, zUtf8Name);
       fossil_mbcs_free(zUtf8Name);
       if( file_isdir(zSubpath)==1 ){
@@ -870,7 +860,7 @@ void recon_read_dir(char *zPath){
     fossil_panic("encountered error %d while trying to open \"%s\".",
                   errno, g.argv[3]);
   }
-  fossil_mbcs_free(zMbcsPath);
+  fossil_mbcs_free(zUnicodePath);
 }
 
 /*
@@ -898,7 +888,7 @@ void reconstruct_cmd(void) {
   db_open_repository(g.argv[2]);
   db_open_config(0);
   db_begin_transaction();
-  db_initial_setup(0, 0, 1);
+  db_initial_setup(0, 0, 0, 1);
 
   fossil_print("Reading files from directory \"%s\"...\n", g.argv[3]);
   recon_read_dir(g.argv[3]);
@@ -937,6 +927,7 @@ void reconstruct_cmd(void) {
 **   -R|--repository REPOSITORY  deconstruct given REPOSITORY
 **   -L|--prefixlength N         set the length of the names of the DESTINATION
 **                               subdirectories to N
+**   --private                   Include private artifacts.
 **
 ** See also: rebuild, reconstruct
 */
@@ -944,16 +935,8 @@ void deconstruct_cmd(void){
   const char *zDestDir;
   const char *zPrefixOpt;
   Stmt        s;
+  int privateFlag;
 
-  /* check number of arguments */
-  if( (g.argc != 3) && (g.argc != 5)  && (g.argc != 7)){
-    usage ("?-R|--repository REPOSITORY? ?-L|--prefixlength N? DESTINATION");
-  }
-  /* get and check argument destination directory */
-  zDestDir = g.argv[g.argc-1];
-  if( !*zDestDir  || !file_isdir(zDestDir)) {
-    fossil_panic("DESTINATION(%s) is not a directory!",zDestDir);
-  }
   /* get and check prefix length argument and build format string */
   zPrefixOpt=find_option("prefixlength","L",1);
   if( !zPrefixOpt ){
@@ -965,6 +948,19 @@ void deconstruct_cmd(void){
       fossil_fatal("N(%s) is not a a valid prefix length!",zPrefixOpt);
     }
   }
+  /* open repository and open query for all artifacts */
+  db_find_and_open_repository(OPEN_ANY_SCHEMA, 0);
+  privateFlag = find_option("private",0,0)!=0;
+  verify_all_options();
+  /* check number of arguments */
+  if( g.argc!=3 ){
+    usage ("?OPTIONS? DESTINATION");
+  }
+  /* get and check argument destination directory */
+  zDestDir = g.argv[g.argc-1];
+  if( !*zDestDir  || !file_isdir(zDestDir)) {
+    fossil_fatal("DESTINATION(%s) is not a directory!",zDestDir);
+  }
 #ifndef _WIN32
   if( file_access(zDestDir, W_OK) ){
     fossil_fatal("DESTINATION(%s) is not writeable!",zDestDir);
@@ -979,8 +975,7 @@ void deconstruct_cmd(void){
   }else{
     zFNameFormat = mprintf("%s/%%s",zDestDir);
   }
-  /* open repository and open query for all artifacts */
-  db_find_and_open_repository(OPEN_ANY_SCHEMA, 0);
+
   bag_init(&bagDone);
   ttyOutput = 1;
   processCnt = 0;
@@ -992,7 +987,8 @@ void deconstruct_cmd(void){
   db_prepare(&s,
      "SELECT rid, size FROM blob /*scan*/"
      " WHERE NOT EXISTS(SELECT 1 FROM shun WHERE uuid=blob.uuid)"
-     "   AND NOT EXISTS(SELECT 1 FROM delta WHERE rid=blob.rid)"
+     "   AND NOT EXISTS(SELECT 1 FROM delta WHERE rid=blob.rid) %s",
+     privateFlag==0 ? "AND rid NOT IN private" : ""
   );
   while( db_step(&s)==SQLITE_ROW ){
     int rid = db_column_int(&s, 0);
@@ -1006,7 +1002,8 @@ void deconstruct_cmd(void){
   db_finalize(&s);
   db_prepare(&s,
      "SELECT rid, size FROM blob"
-     " WHERE NOT EXISTS(SELECT 1 FROM shun WHERE uuid=blob.uuid)"
+     " WHERE NOT EXISTS(SELECT 1 FROM shun WHERE uuid=blob.uuid) %s",
+     privateFlag==0 ? "AND rid NOT IN private" : ""
   );
   while( db_step(&s)==SQLITE_ROW ){
     int rid = db_column_int(&s, 0);
