@@ -47,6 +47,93 @@ static int headerHasBeenGenerated = 0;
 */
 static int sideboxUsed = 0;
 
+
+/*
+** List of hyperlinks that need to be resolved by javascript in
+** the footer.
+*/
+char **aHref = 0;
+int nHref = 0;
+int nHrefAlloc = 0;
+
+/*
+** Generate and return a anchor tag like this:
+**
+**        <a href="URL">
+**  or    <a id="ID">
+**
+** The form of the anchor tag is determined by the g.javascriptHyperlink
+** variable.  The href="URL" form is used if g.javascriptHyperlink is false.
+** If g.javascriptHyperlink is true then the
+** id="ID" form is used and javascript is generated in the footer to cause
+** href values to be inserted after the page has loaded.  If 
+** g.perm.History is false, then the <a id="ID"> form is still
+** generated but the javascript is not generated so the links never
+** activate.
+**
+** Filling in the href="URL" using javascript is a defense against bots.
+**
+** The name of this routine is deliberately kept short so that can be
+** easily used within @-lines.  Example:
+**
+**      @ %z(href("%R/artifact/%s",zUuid))%h(zFN)</a>
+**
+** Note %z format.  The string returned by this function is always
+** obtained from fossil_malloc() so rendering it with %z will reclaim
+** that memory space.
+**
+** There are two versions of this routine: href() does a plain hyperlink
+** and xhref() adds extra attribute text.
+*/
+char *xhref(const char *zExtra, const char *zFormat, ...){
+  char *zUrl;
+  va_list ap;
+  va_start(ap, zFormat);
+  zUrl = vmprintf(zFormat, ap);
+  va_end(ap);
+  if( g.perm.Hyperlink && !g.javascriptHyperlink ){
+    return mprintf("<a %s href=\"%z\">", zExtra, zUrl);
+  }
+  if( nHref>=nHrefAlloc ){
+    nHrefAlloc = nHrefAlloc*2 + 10;
+    aHref = fossil_realloc(aHref, nHrefAlloc*sizeof(aHref[0]));
+  }
+  aHref[nHref++] = zUrl;
+  return mprintf("<a %s id=%d>", zExtra, nHref);
+}
+char *href(const char *zFormat, ...){
+  char *zUrl;
+  va_list ap;
+  va_start(ap, zFormat);
+  zUrl = vmprintf(zFormat, ap);
+  va_end(ap);
+  if( g.perm.Hyperlink && !g.javascriptHyperlink ){
+    return mprintf("<a href=\"%z\">", zUrl);
+  }
+  if( nHref>=nHrefAlloc ){
+    nHrefAlloc = nHrefAlloc*2 + 10;
+    aHref = fossil_realloc(aHref, nHrefAlloc*sizeof(aHref[0]));
+  }
+  aHref[nHref++] = zUrl;
+  return mprintf("<a id=%d>", nHref);
+}
+
+/*
+** Generate javascript that will set the href= attribute on all anchors.
+*/
+void style_resolve_href(void){
+  int i;
+  if( !g.perm.Hyperlink || !g.javascriptHyperlink || nHref==0 ) return;
+  @ <script type="text/JavaScript">
+  @ /* <![CDATA[ */
+  @ function u(i,h){gebi(i).href=h;}
+  for(i=0; i<nHref; i++){
+    @ u(%d(i+1),"%s(aHref[i])");
+  }
+  @ /* ]]> */
+  @ </script>
+}
+
 /*
 ** Add a new element to the submenu
 */
@@ -75,6 +162,26 @@ static int submenuCompare(const void *a, const void *b){
   return fossil_strcmp(A->zLabel, B->zLabel);
 }
 
+/* Use this for the $current_page variable if it is not NULL.  If it is
+** NULL then use g.zPath.
+*/
+static char *local_zCurrentPage = 0;
+
+/*
+** Set the desired $current_page to something other than g.zPath
+*/
+void style_set_current_page(const char *zFormat, ...){
+  fossil_free(local_zCurrentPage);
+  if( zFormat==0 ){
+    local_zCurrentPage = 0;
+  }else{
+    va_list ap;
+    va_start(ap, zFormat);
+    local_zCurrentPage = vmprintf(zFormat, ap);
+    va_end(ap);
+  }
+}
+
 /*
 ** Draw the header.
 */
@@ -89,7 +196,8 @@ void style_header(const char *zTitleFormat, ...){
   va_end(ap);
   
   cgi_destination(CGI_HEADER);
-  cgi_printf("%s","<!DOCTYPE html>");
+
+  @ <!DOCTYPE html>
   
   if( g.thTrace ) Th_Trace("BEGIN_HEADER<br />\n", -1);
 
@@ -99,7 +207,7 @@ void style_header(const char *zTitleFormat, ...){
   Th_Store("baseurl", g.zBaseURL);
   Th_Store("home", g.zTop);
   Th_Store("index_page", db_get("index-page","/home"));
-  Th_Store("current_page", g.zPath);
+  Th_Store("current_page", local_zCurrentPage ? local_zCurrentPage : g.zPath);
   Th_Store("release_version", RELEASE_VERSION);
   Th_Store("manifest_version", MANIFEST_VERSION);
   Th_Store("manifest_date", MANIFEST_DATE);
@@ -115,6 +223,39 @@ void style_header(const char *zTitleFormat, ...){
   g.cgiOutput = 1;
   headerHasBeenGenerated = 1;
   sideboxUsed = 0;
+
+  /* Make the gebi(x) function available as an almost-alias for
+  ** document.getElementById(x) (except that it throws an error
+  ** if the element is not found).
+  **
+  ** Maintenance note: this function must of course be available
+  ** before it is called. It "should" go in the HEAD so that client
+  ** HEAD code can make use of it, but because the client can replace
+  ** the HEAD, and some fossil pages rely on gebi(), we put it here.
+  */
+  @ <script>
+  @ function gebi(x){
+  @ if(/^#/.test(x)) x = x.substr(1);
+  @ var e = document.getElementById(x);
+  @ if(!e) throw new Error("Expecting element with ID "+x);
+  @ else return e;}
+  @ </script>
+}
+
+/*
+** Append ad unit text if appropriate.
+*/
+static void style_ad_unit(void){
+  const char *zAd;
+  if( g.perm.Admin && db_get_boolean("adunit-omit-if-admin",0) ){
+    return;
+  }
+  if( g.zLogin && strcmp(g.zLogin,"anonymous")!=0
+      && db_get_boolean("adunit-omit-if-user",0) ){
+    return;
+  }
+  zAd = db_get("adunit", 0);
+  if( zAd ) cgi_append_content(zAd, -1);
 }
 
 /*
@@ -144,6 +285,7 @@ void style_footer(void){
     }
     @ </div>
   }
+  style_ad_unit();
   @ <div class="content">
   cgi_destination(CGI_BODY);
 
@@ -155,6 +297,11 @@ void style_footer(void){
     @ <div class="endContent"></div>
   }
   @ </div>
+
+  /* Set the href= field on hyperlinks.  Do this before the footer since
+  ** the footer will be generating </html> */
+  style_resolve_href();
+
   zFooter = db_get("footer", (char*)zDefaultFooter);
   if( g.thTrace ) Th_Trace("BEGIN_FOOTER<br />\n", -1);
   Th_Render(zFooter);
@@ -192,6 +339,7 @@ void style_sidebox_end(void){
 const char zDefaultHeader[] = 
 @ <html>
 @ <head>
+@ <base href="$baseurl/$current_page" />
 @ <title>$<project_name>: $<title></title>
 @ <link rel="alternate" type="application/rss+xml" title="RSS Feed"
 @       href="$home/timeline.rss" />
@@ -216,30 +364,30 @@ const char zDefaultHeader[] =
 @ <th1>
 @ html "<a href='$home$index_page'>Home</a>\n"
 @ if {[anycap jor]} {
-@   html "<a href='$home/timeline'>Timeline</a>\n"
+@   html "<a href='timeline'>Timeline</a>\n"
 @ }
 @ if {[hascap oh]} {
-@   html "<a href='$home/dir?ci=tip'>Files</a>\n"
+@   html "<a href='dir?ci=tip'>Files</a>\n"
 @ }
 @ if {[hascap o]} {
-@   html "<a href='$home/brlist'>Branches</a>\n"
-@   html "<a href='$home/taglist'>Tags</a>\n"
+@   html "<a href='brlist'>Branches</a>\n"
+@   html "<a href='taglist'>Tags</a>\n"
 @ }
 @ if {[hascap r]} {
-@   html "<a href='$home/reportlist'>Tickets</a>\n"
+@   html "<a href='reportlist'>Tickets</a>\n"
 @ }
 @ if {[hascap j]} {
-@   html "<a href='$home/wiki'>Wiki</a>\n"
+@   html "<a href='wiki'>Wiki</a>\n"
 @ }
 @ if {[hascap s]} {
-@   html "<a href='$home/setup'>Admin</a>\n"
+@   html "<a href='setup'>Admin</a>\n"
 @ } elseif {[hascap a]} {
-@   html "<a href='$home/setup_ulist'>Users</a>\n"
+@   html "<a href='setup_ulist'>Users</a>\n"
 @ }
 @ if {[info exists login]} {
-@   html "<a href='$home/login'>Logout</a>\n"
+@   html "<a href='login'>Logout</a>\n"
 @ } else {
-@   html "<a href='$home/login'>Login</a>\n"
+@   html "<a href='login'>Login</a>\n"
 @ }
 @ </th1></div>
 ;
@@ -319,6 +467,8 @@ const char zDefaultCSS[] =
 @   text-align: center;
 @   letter-spacing: 1px;
 @   background-color: #558195;
+@   border-top-left-radius: 8px;
+@   border-top-right-radius: 8px;
 @   color: white;
 @ }
 @
@@ -344,7 +494,9 @@ const char zDefaultCSS[] =
 @ /* All page content from the bottom of the menu or submenu down to
 @ ** the footer */
 @ div.content {
-@   padding: 0ex 1ex 0ex 2ex;
+@   padding: 0ex 1ex 1ex 1ex;
+@   border: solid #aaa;
+@   border-width: 1px;
 @ }
 @
 @ /* Some pages have section dividers */
@@ -375,10 +527,11 @@ const char zDefaultCSS[] =
 @ div.footer {
 @   clear: both;
 @   font-size: 0.8em;
-@   margin-top: 12px;
 @   padding: 5px 10px 5px 10px;
 @   text-align: right;
 @   background-color: #558195;
+@   border-bottom-left-radius: 8px;
+@   border-bottom-right-radius: 8px;
 @   color: white;
 @ }
 @
@@ -422,7 +575,7 @@ const struct strctCssDefaults {
     @   background-color: white;
     @   border-width: medium;
     @   border-style: double;
-    @   margin: 10;
+    @   margin: 10px;
   },
   { "div.sideboxTitle",
     "The nomenclature title in sideboxes for branches,..",
@@ -445,14 +598,12 @@ const struct strctCssDefaults {
   },
   { "table.timelineTable",
     "the format for the timeline data table",
-    @   cellspacing: 0;
     @   border: 0;
-    @   cellpadding: 0
   },
   { "td.timelineTableCell",
     "the format for the timeline data cells",
-    @   valign: top;
-    @   align: left;
+    @   vertical-align: top;
+    @   text-align: left;
   },
   { "span.timelineLeaf",
     "the format for the timeline leaf marks",
@@ -815,24 +966,30 @@ void cgi_append_default_css(void) {
 ** WEBPAGE: style.css
 */
 void page_style_css(void){
-  const char *zCSS    = 0;
+  Blob css;
   int i;
 
   cgi_set_content_type("text/css");
-  zCSS = db_get("css",(char*)zDefaultCSS);
-  /* append user defined css */
-  cgi_append_content(zCSS, -1);
+  blob_init(&css, db_get("css",(char*)zDefaultCSS), -1);
+
   /* add special missing definitions */
-  for (i=1;cssDefaultList[i].elementClass;i++)
-    if (!strstr(zCSS,cssDefaultList[i].elementClass)) {
-      cgi_append_content("/* ", -1);
-      cgi_append_content(cssDefaultList[i].comment, -1);
-      cgi_append_content(" */\n", -1);
-      cgi_append_content(cssDefaultList[i].elementClass, -1);
-      cgi_append_content(" {\n", -1);
-      cgi_append_content(cssDefaultList[i].value, -1);
-      cgi_append_content("}\n\n", -1);
+  for(i=1; cssDefaultList[i].elementClass; i++){
+    if( strstr(blob_str(&css), cssDefaultList[i].elementClass)==0 ){
+      blob_appendf(&css, "/* %s */\n%s {\n%s}\n",
+          cssDefaultList[i].comment,
+          cssDefaultList[i].elementClass,
+          cssDefaultList[i].value);
     }
+  }
+
+  /* Process through TH1 in order to give an opportunity to substitute
+  ** variables such as $baseurl.
+  */
+  Th_Store("baseurl", g.zBaseURL);
+  Th_Store("home", g.zTop);
+  Th_Render(blob_str(&css));
+
+  /* Tell CGI that the content returned by this page is considered cacheable */
   g.isConst = 1;
 }
 
