@@ -49,8 +49,8 @@ static void shorten_uuid(char *zDest, const char *zSrc){
 void hyperlink_to_uuid(const char *zUuid){
   char z[UUID_SIZE+1];
   shorten_uuid(z, zUuid);
-  if( g.perm.History ){
-    @ <a class="timelineHistLink" href="%s(g.zTop)/info/%s(z)">[%s(z)]</a>
+  if( g.perm.Hyperlink ){
+    @ %z(xhref("class='timelineHistLink'","%R/info/%s",z))[%s(z)]</a>
   }else{
     @ <span class="timelineHistDsp">[%s(z)]</span>
   }
@@ -60,11 +60,11 @@ void hyperlink_to_uuid(const char *zUuid){
 ** Generate a hyperlink to a diff between two versions.
 */
 void hyperlink_to_diff(const char *zV1, const char *zV2){
-  if( g.perm.History ){
+  if( g.perm.Hyperlink ){
     if( zV2==0 ){
-      @ <a href="%s(g.zTop)/diff?v2=%s(zV1)">[diff]</a>
+      @ %z(href("%R/diff?v2=%s",zV1))[diff]</a>
     }else{
-      @ <a href="%s(g.zTop)/diff?v1=%s(zV1)&amp;v2=%s(zV2)">[diff]</a>
+      @ %z(href("%R/diff?v1=%s&v2=%s",zV1,zV2))[diff]</a>
     }
   }
 }
@@ -74,8 +74,8 @@ void hyperlink_to_diff(const char *zV1, const char *zV2){
 */
 void hyperlink_to_date(const char *zDate, const char *zSuffix){
   if( zSuffix==0 ) zSuffix = "";
-  if( g.perm.History ){
-    @ <a href="%s(g.zTop)/timeline?c=%T(zDate)">%s(zDate)</a>%s(zSuffix)
+  if( g.perm.Hyperlink ){
+    @ %z(href("%R/timeline?c=%T",zDate))%s(zDate)</a>%s(zSuffix)
   }else{
     @ %s(zDate)%s(zSuffix)
   }
@@ -88,11 +88,11 @@ void hyperlink_to_date(const char *zDate, const char *zSuffix){
 */
 void hyperlink_to_user(const char *zU, const char *zD, const char *zSuf){
   if( zSuf==0 ) zSuf = "";
-  if( g.perm.History ){
+  if( g.perm.Hyperlink ){
     if( zD && zD[0] ){
-      @ <a href="%s(g.zTop)/timeline?c=%T(zD)&amp;u=%T(zU)">%h(zU)</a>%s(zSuf)
+      @ %z(href("%R/timeline?c=%T&u=%T",zD,zU))%h(zU)</a>%s(zSuf)
     }else{
-      @ <a href="%s(g.zTop)/timeline?u=%T(zU)">%h(zU)</a>%s(zSuf)
+      @ %z(href("%R/timeline?u=%T",zU))%h(zU)</a>%s(zSuf)
     }
   }else{
     @ %s(zU)
@@ -355,25 +355,23 @@ void www_print_timeline(
     ** with a hyperlink to another timeline for that user.
     */
     if( zTagList && zTagList[0]==0 ) zTagList = 0;
-    if( g.perm.History && fossil_strcmp(zUser, zThisUser)!=0 ){
-      char *zLink = mprintf("%s/timeline?u=%h&c=%t&nd",
-                            g.zTop, zUser, zDate);
-      @ (user: <a href="%s(zLink)">%h(zUser)</a>%s(zTagList?",":"\051")
-      fossil_free(zLink);
+    if( g.perm.Hyperlink && fossil_strcmp(zUser, zThisUser)!=0 ){
+      char *zLink = mprintf("%R/timeline?u=%h&c=%t&nd", zUser, zDate);
+      @ (user: %z(href("%z",zLink))%h(zUser)</a>%s(zTagList?",":"\051")
     }else{
       @ (user: %h(zUser)%s(zTagList?",":"\051")
     }
 
     /* Generate a "detail" link for tags. */
-    if( zType[0]=='g' && g.perm.History ){
-      @ [<a href="%s(g.zTop)/info/%S(zUuid)">details</a>]
+    if( zType[0]=='g' && g.perm.Hyperlink ){
+      @ [%z(href("%R/info/%S",zUuid))details</a>]
     }
 
     /* Generate the "tags: TAGLIST" at the end of the comment, together
     ** with hyperlinks to the tag list.
     */
     if( zTagList ){
-      if( g.perm.History ){
+      if( g.perm.Hyperlink ){
         int i;
         const char *z = zTagList;
         Blob links;
@@ -382,8 +380,8 @@ void www_print_timeline(
           for(i=0; z[i] && (z[i]!=',' || z[i+1]!=' '); i++){}
           if( zThisTag==0 || memcmp(z, zThisTag, i)!=0 || zThisTag[i]!=0 ){
             blob_appendf(&links,
-                  "<a href=\"%s/timeline?r=%#t&nd&c=%s\">%#h</a>%.2s",
-                  g.zTop, i, z, zDate, i, z, &z[i]
+                  "%z%#h</a>%.2s",
+                  href("%R/timeline?r=%#t&nd&c=%s",i,z,zDate), i,z, &z[i]
             );
           }else{
             blob_appendf(&links, "%#h", i+2, z);
@@ -405,7 +403,7 @@ void www_print_timeline(
     }
 
     /* Generate the file-change list if requested */
-    if( (tmFlags & TIMELINE_FCHANGES)!=0 && zType[0]=='c' && g.perm.History ){
+    if( (tmFlags & TIMELINE_FCHANGES)!=0 && zType[0]=='c' && g.perm.Hyperlink ){
       int inUl = 0;
       if( !fchngQueryInit ){
         db_prepare(&fchngQuery, 
@@ -435,22 +433,22 @@ void www_print_timeline(
         }
         if( isNew ){
           @ <li> %h(zFilename) (new file) &nbsp;
-          @ <a href="%s(g.zTop)/artifact/%S(zNew)"
-          @ target="diffwindow">[view]</a></li>
+          @ %z(xhref("target='diffwindow'","%R/artifact/%S",zNew))
+          @ [view]</a></li>
         }else if( isDel ){
           @ <li> %h(zFilename) (deleted)</li>
         }else if( fossil_strcmp(zOld,zNew)==0 && zOldName!=0 ){
           @ <li> %h(zOldName) &rarr; %h(zFilename)
-          @ <a href="%s(g.zTop)/artifact/%S(zNew)"
-          @ target="diffwindow">[view]</a></li>
+          @ %z(xhref("target='diffwindow'","%R/artifact/%S",zNew))
+          @ [view]</a></li>
         }else{
           if( zOldName!=0 ){
             @ <li> %h(zOldName) &rarr; %h(zFilename)
           }else{
             @ <li> %h(zFilename) &nbsp;
           }
-          @ <a href="%s(g.zTop)/fdiff?v1=%S(zOld)&v2=%S(zNew)"
-          @ target="diffwindow">[diff]</a></li>
+          @ %z(xhref("target='diffwindow'","%R/fdiff?v1=%S&v2=%S",zOld,zNew))
+          @ [diff]</a></li>
         }
       }
       db_reset(&fchngQuery);
@@ -571,7 +569,7 @@ void timeline_output_graph_javascript(GraphContext *pGraph, int omitDescenders){
     }
     cgi_printf("var nrail = %d\n", pGraph->mxRail+1);
     graph_free(pGraph);
-    @ var canvasDiv = document.getElementById("canvas");
+    @ var canvasDiv = gebi("canvas");
 #if 0
     @ var realCanvas = null;
 #endif
@@ -591,7 +589,7 @@ void timeline_output_graph_javascript(GraphContext *pGraph, int omitDescenders){
     @   canvasDiv.appendChild(n);
     @ }
     @ function absoluteY(id){
-    @   var obj = document.getElementById(id);
+    @   var obj = gebi(id);
     @   if( !obj ) return;
     @   var top = 0;
     @   if( obj.offsetParent ){
@@ -602,7 +600,7 @@ void timeline_output_graph_javascript(GraphContext *pGraph, int omitDescenders){
     @   return top;
     @ }
     @ function absoluteX(id){
-    @   var obj = document.getElementById(id);
+    @   var obj = gebi(id);
     @   if( !obj ) return;
     @   var left = 0;
     @   if( obj.offsetParent ){
@@ -691,7 +689,7 @@ void timeline_output_graph_javascript(GraphContext *pGraph, int omitDescenders){
     @   }
     @ }
     @ function renderGraph(){
-    @   var canvasDiv = document.getElementById("canvas");
+    @   var canvasDiv = gebi("canvas");
     @   while( canvasDiv.hasChildNodes() ){
     @     canvasDiv.removeChild(canvasDiv.firstChild);
     @   }
@@ -708,7 +706,7 @@ void timeline_output_graph_javascript(GraphContext *pGraph, int omitDescenders){
     @     canvasDiv.innerHTML = '<canvas id="timeline-canvas" '+
     @        'style="position:absolute;left:'+(left-5)+'px;"' +
     @        ' width="'+width+'" height="'+btm+'"><'+'/canvas>';
-    @     realCanvas = document.getElementById('timeline-canvas');
+    @     realCanvas = gebi('timeline-canvas');
     @   }else{
     @     realCanvas = 0;
     @   }
@@ -814,29 +812,49 @@ static void timeline_submenu(
 }
 
 
+/*
+** Convert a symbolic name used as an argument to the a=, b=, or c=
+** query parameters of timeline into a julianday mtime value.
+*/
+double symbolic_name_to_mtime(const char *z){
+  double mtime;
+  int rid;
+  if( z==0 ) return -1.0;
+  if( fossil_isdate(z) ){
+    mtime = db_double(0.0, "SELECT julianday(%Q,'utc')", z);
+    if( mtime>0.0 ) return mtime;
+  }
+  rid = symbolic_name_to_rid(z, "ci");
+  if( rid==0 ) return -1.0;
+  mtime = db_double(0.0, "SELECT mtime FROM event WHERE objid=%d", rid);
+  return mtime;
+}
+
+/*
+** The value of one second in julianday notation
+*/
+#define ONE_SECOND (1.0/86400.0)
+
 /*
 ** zDate is a localtime date.  Insert records into the
 ** "timeline" table to cause <hr> to be inserted before and after
 ** entries of that date.  If zDate==NULL then put dividers around
 ** the event identified by rid.
 */
-static void timeline_add_dividers(const char *zDate, int rid){
+static void timeline_add_dividers(double rDate, int rid){
   char *zToDel = 0;
-  if( zDate==0 ){
-    zToDel = db_text(0,"SELECT julianday(mtime,'localtime') FROM event"
-                       " WHERE objid=%d", rid);
-    zDate = zToDel;
-    if( zDate==0 ) zDate = "1";
+  if( rDate==0 ){
+    rDate = db_double(0.0, "SELECT mtime FROM event WHERE objid=%d", rid);
   }
   db_multi_exec(
     "INSERT INTO timeline(rid,sortby,etype)"
-    "VALUES(-1,julianday(%Q,'utc')-1.0e-5,'div')",
-    zDate
+    "VALUES(-1,%.16g,'div')",
+    rDate-ONE_SECOND
   );
   db_multi_exec(
     "INSERT INTO timeline(rid,sortby,etype)"
-    "VALUES(-2,julianday(%Q,'utc')+1.0e-5,'div')",
-     zDate
+    "VALUES(-2,%.17g,'div')",
+    rDate+ONE_SECOND
   );
   fossil_free(zToDel);
 }
@@ -847,10 +865,10 @@ static void timeline_add_dividers(const char *zDate, int rid){
 **
 ** Query parameters:
 **
-**    a=TIMESTAMP    after this date
-**    b=TIMESTAMP    before this date.
-**    c=TIMESTAMP    "circa" this date.
-**    n=COUNT        number of events in output
+**    a=TIMEORTAG    after this event
+**    b=TIMEORTAG    before this event
+**    c=TIMEORTAG    "circa" this event
+**    n=COUNT        max number of events in output
 **    p=UUID         artifact and up to COUNT parents and ancestors
 **    d=UUID         artifact and up to COUNT descendants
 **    dp=UUUID       The same as d=UUID&p=UUID
@@ -901,10 +919,11 @@ void page_timeline(void){
   HQuery url;                        /* URL for various branch links */
   int from_rid = name_to_typed_rid(P("from"),"ci"); /* from= for paths */
   int to_rid = name_to_typed_rid(P("to"),"ci");    /* to= for path timelines */
-  int noMerge = P("nomerge")!=0;          /* Do not follow merge links */
+  int noMerge = P("shortest")==0;           /* Follow merge links if shorter */
   int me_rid = name_to_typed_rid(P("me"),"ci");  /* me= for common ancestory */
   int you_rid = name_to_typed_rid(P("you"),"ci");/* you= for common ancst */
   int pd_rid;
+  double rBefore, rAfter, rCirca;     /* Boundary times */
 
   /* To view the timeline, must have permission to read project data.
   */
@@ -976,17 +995,9 @@ void page_timeline(void){
     blob_append(&sql, ")", -1);
     path_reset();
     blob_append(&desc, "All nodes on the path from ", -1);
-    if( g.perm.History ){
-      blob_appendf(&desc, "<a href='%s/info/%h'>[%h]</a>",  g.zTop,zFrom,zFrom);
-    }else{
-      blob_appendf(&desc, "[%h]", zFrom);
-    }
+    blob_appendf(&desc, "%z%h</a>", href("%R/info/%h", zFrom), zFrom);
     blob_append(&desc, " and ", -1);
-    if( g.perm.History ){
-      blob_appendf(&desc, "<a href='%s/info/%h'>[%h]</a>.",  g.zTop, zTo, zTo);
-    }else{
-      blob_appendf(&desc, "[%h].", zTo);
-    }
+    blob_appendf(&desc, "%z[%h]</a>", href("%R/info/%h",zTo), zTo);
     tmFlags |= TIMELINE_DISJOINT;
     db_multi_exec("%s", blob_str(&sql));
   }else if( (p_rid || d_rid) && g.perm.Read ){
@@ -1014,7 +1025,7 @@ void page_timeline(void){
       db_multi_exec("DELETE FROM ok");
     }
     if( p_rid ){
-      compute_ancestors(p_rid, nEntry+1);
+      compute_ancestors(p_rid, nEntry+1, 0);
       np = db_int(0, "SELECT count(*)-1 FROM ok");
       if( np>0 ){
         if( nd>0 ) blob_appendf(&desc, " and ");
@@ -1023,12 +1034,8 @@ void page_timeline(void){
       }
       if( d_rid==0 && useDividers ) timeline_add_dividers(0, p_rid);
     }
-    if( g.perm.History ){
-      blob_appendf(&desc, " of <a href='%s/info/%s'>[%.10s]</a>",
-                   g.zTop, zUuid, zUuid);
-    }else{
-      blob_appendf(&desc, " of check-in [%.10s]", zUuid);
-    }
+    blob_appendf(&desc, " of %z[%.10s]</a>",
+                   href("%R/info/%s", zUuid), zUuid);
   }else if( f_rid && g.perm.Read ){
     /* If f= is present, ignore all other parameters other than n= */
     char *zUuid;
@@ -1044,12 +1051,7 @@ void page_timeline(void){
     if( useDividers ) timeline_add_dividers(0, f_rid);
     blob_appendf(&desc, "Parents and children of check-in ");
     zUuid = db_text("", "SELECT uuid FROM blob WHERE rid=%d", f_rid);
-    if( g.perm.History ){
-      blob_appendf(&desc, "<a href='%s/info/%s'>[%.10s]</a>",
-                   g.zTop, zUuid, zUuid);
-    }else{
-      blob_appendf(&desc, "[%.10s]", zUuid);
-    }
+    blob_appendf(&desc, "%z[%.10s]</a>", href("%R/info/%s", zUuid), zUuid);
   }else{
     /* Otherwise, a timeline based on a span of time */
     int n;
@@ -1142,59 +1144,51 @@ void page_timeline(void){
         zSearch, zSearch);
       url_add_parameter(&url, "s", zSearch);
     }
-    if( zAfter ){
-      while( fossil_isspace(zAfter[0]) ){ zAfter++; }
-      if( zAfter[0] ){
+    rBefore = symbolic_name_to_mtime(zBefore);
+    rAfter = symbolic_name_to_mtime(zAfter);
+    rCirca = symbolic_name_to_mtime(zCirca);
+    if( rAfter>0.0 ){
+      if( rBefore>0.0 ){
         blob_appendf(&sql, 
-           " AND event.mtime>=(SELECT julianday(%Q, 'utc'))"
-           " ORDER BY event.mtime ASC", zAfter);
+           " AND event.mtime>=%.17g AND event.mtime<=%.17g"
+           " ORDER BY event.mtime ASC", rAfter-ONE_SECOND, rBefore+ONE_SECOND);
         url_add_parameter(&url, "a", zAfter);
-        zBefore = 0;
-      }else{
-        zAfter = 0;
-      }
-    }else if( zBefore ){
-      while( fossil_isspace(zBefore[0]) ){ zBefore++; }
-      if( zBefore[0] ){
-        blob_appendf(&sql, 
-           " AND event.mtime<=(SELECT julianday(%Q, 'utc'))"
-           " ORDER BY event.mtime DESC", zBefore);
         url_add_parameter(&url, "b", zBefore);
-       }else{
-        zBefore = 0;
-      }
-    }else if( zCirca ){
-      while( fossil_isspace(zCirca[0]) ){ zCirca++; }
-      if( zCirca[0] ){
-        double rCirca = db_double(0.0, "SELECT julianday(%Q, 'utc')", zCirca);
-        Blob sql2;
-        blob_init(&sql2, blob_str(&sql), -1);
-        blob_appendf(&sql2,
-            " AND event.mtime<=%f ORDER BY event.mtime DESC LIMIT %d",
-            rCirca, (nEntry+1)/2
-        );
-        db_multi_exec("%s", blob_str(&sql2));
-        blob_reset(&sql2);
-        blob_appendf(&sql,
-            " AND event.mtime>=%f ORDER BY event.mtime ASC",
-            rCirca
-        );
-        nEntry -= (nEntry+1)/2;
-        if( useDividers ) timeline_add_dividers(zCirca, 0);
-        url_add_parameter(&url, "c", zCirca);
+        nEntry = 1000000;
       }else{
-        zCirca = 0;
+        blob_appendf(&sql, 
+           " AND event.mtime>=%.17g  ORDER BY event.mtime ASC",
+           rAfter-ONE_SECOND);
+        url_add_parameter(&url, "a", zAfter);
       }
+    }else if( rBefore>0.0 ){
+      blob_appendf(&sql, 
+         " AND event.mtime<=%.17g ORDER BY event.mtime DESC",
+         rBefore+ONE_SECOND);
+      url_add_parameter(&url, "b", zBefore);
+    }else if( rCirca>0.0 ){
+      Blob sql2;
+      blob_init(&sql2, blob_str(&sql), -1);
+      blob_appendf(&sql2,
+          " AND event.mtime<=%f ORDER BY event.mtime DESC LIMIT %d",
+          rCirca, (nEntry+1)/2
+      );
+      db_multi_exec("%s", blob_str(&sql2));
+      blob_reset(&sql2);
+      blob_appendf(&sql,
+          " AND event.mtime>=%f ORDER BY event.mtime ASC",
+          rCirca
+      );
+      nEntry -= (nEntry+1)/2;
+      if( useDividers ) timeline_add_dividers(rCirca, 0);
+      url_add_parameter(&url, "c", zCirca);
     }else{
       blob_appendf(&sql, " ORDER BY event.mtime DESC");
     }
     blob_appendf(&sql, " LIMIT %d", nEntry);
     db_multi_exec("%s", blob_str(&sql));
 
-    n = db_int(0, "SELECT count(*) FROM timeline /*scan*/");
-    if( n<nEntry && zAfter ){
-      cgi_redirect(url_render(&url, "a", 0, "b", 0));
-    }
+    n = db_int(0, "SELECT count(*) FROM timeline WHERE etype!='div' /*scan*/");
     if( zAfter==0 && zBefore==0 && zCirca==0 ){
       blob_appendf(&desc, "%d most recent %ss", n, zEType);
     }else{
@@ -1211,17 +1205,22 @@ void page_timeline(void){
       blob_appendf(&desc, " related to \"%h\"", zBrName);
       tmFlags |= TIMELINE_DISJOINT;
     }
-    if( zAfter ){
-      blob_appendf(&desc, " occurring on or after %h.<br />", zAfter);
-    }else if( zBefore ){
+    if( rAfter>0.0 ){
+      if( rBefore>0.0 ){
+        blob_appendf(&desc, " occurring between %h and %h.<br>",
+                     zAfter, zBefore);
+      }else{
+        blob_appendf(&desc, " occurring on or after %h.<br />", zAfter);
+      }
+    }else if( rBefore>0.0 ){
       blob_appendf(&desc, " occurring on or before %h.<br />", zBefore);
-    }else if( zCirca ){
+    }else if( rCirca>0.0 ){
       blob_appendf(&desc, " occurring around %h.<br />", zCirca);
     }
     if( zSearch ){
       blob_appendf(&desc, " matching \"%h\"", zSearch);
     }
-    if( g.perm.History ){
+    if( g.perm.Hyperlink ){
       if( zAfter || n==nEntry ){
         zDate = db_text(0, "SELECT min(timestamp) FROM timeline /*scan*/");
         timeline_submenu(&url, "Older", "b", zDate, "a");
@@ -1541,7 +1540,7 @@ void timeline_cmd(void){
     if( mode==3 ){
       compute_descendants(objid, n);
     }else{
-      compute_ancestors(objid, n);
+      compute_ancestors(objid, n, 0);
     }
     blob_appendf(&sql, " AND blob.rid IN ok");
   }
@@ -1625,7 +1624,7 @@ void test_timewarp_page(void){
   Stmt q;
 
   login_check_credentials();
-  if( !g.perm.Read || !g.perm.History ){ login_needed(); return; }
+  if( !g.perm.Read || !g.perm.Hyperlink ){ login_needed(); return; }
   style_header("Instances of timewarp");
   @ <ul>
   db_prepare(&q,
@@ -1637,7 +1636,7 @@ void test_timewarp_page(void){
   while( db_step(&q)==SQLITE_ROW ){
     const char *zUuid = db_column_text(&q, 0);
     @ <li>
-    @ <a href="%s(g.zTop)/timeline?p=%S(zUuid)&amp;d=%S(zUuid)">%S(zUuid)</a>
+    @ <a href="%s(g.zTop)/timeline?p=%S(zUuid)&d=%S(zUuid)">%S(zUuid)</a>
   }
   db_finalize(&q);
   style_footer();
