@@ -102,11 +102,13 @@ void update_cmd(void){
   int nochangeFlag;     /* -n or --nochange.  Do a dry run */
   int verboseFlag;      /* -v or --verbose.  Output extra information */
   int debugFlag;        /* --debug option */
+  int setmtimeFlag;     /* --setmtime.  Set mtimes on files */
   int nChng;            /* Number of file renames */
   int *aChng;           /* Array of file renames */
   int i;                /* Loop counter */
   int nConflict = 0;    /* Number of merge conflicts */
   int nOverwrite = 0;   /* Number of unmanaged files overwritten */
+  int nUpdate = 0;      /* Number of chagnes of any kind */
   Stmt mtimeXfer;       /* Statment to transfer mtimes */
 
   if( !internalUpdate ){
@@ -117,18 +119,17 @@ void update_cmd(void){
   nochangeFlag = find_option("nochange","n",0)!=0;
   verboseFlag = find_option("verbose","v",0)!=0;
   debugFlag = find_option("debug",0,0)!=0;
+  setmtimeFlag = find_option("setmtime",0,0)!=0;
   db_must_be_within_tree();
   vid = db_lget_int("checkout", 0);
   if( vid==0 ){
     fossil_fatal("cannot find current version");
   }
-  if( !nochangeFlag && db_exists("SELECT 1 FROM vmerge") ){
-    fossil_fatal("cannot update an uncommitted merge");
-  }
   if( !nochangeFlag && !internalUpdate ) autosync(AUTOSYNC_PULL);
   
-  /* Create any empty directories now, as well as after the update, so changes in settings are reflected now */
-  ensure_empty_dirs_created();
+  /* Create any empty directories now, as well as after the update,
+  ** so changes in settings are reflected now */
+  if( !nochangeFlag ) ensure_empty_dirs_created();
 
   if( internalUpdate ){
     tid = internalUpdate;
@@ -197,7 +198,7 @@ void update_cmd(void){
   }
 
   db_begin_transaction();
-  vfile_check_signature(vid, 1, 0);
+  vfile_check_signature(vid, CKSIG_ENOTFILE);
   if( !nochangeFlag && !internalUpdate ) undo_begin();
   load_vfile_from_rid(tid);
 
@@ -367,6 +368,7 @@ void update_cmd(void){
     zFullPath = mprintf("%s%s", g.zLocalRoot, zName);
     zFullNewPath = mprintf("%s%s", g.zLocalRoot, zNewName);
     nameChng = fossil_strcmp(zName, zNewName);
+    nUpdate++;
     if( idv>0 && ridv==0 && idt>0 && ridt>0 ){
       /* Conflict.  This file has been added to the current checkout
       ** but also exists in the target checkout.  Use the current version.
@@ -450,6 +452,7 @@ void update_cmd(void){
       blob_reset(&t);
       blob_reset(&r);
     }else{
+      nUpdate--;
       if( chnged ){
         if( verboseFlag ) fossil_print("EDITED %s\n", zName);
       }else{
@@ -465,12 +468,35 @@ void update_cmd(void){
   }
   db_finalize(&q);
   db_finalize(&mtimeXfer);
-  fossil_print("--------------\n");
-  show_common_info(tid, "updated-to:", 1, 0);
+  fossil_print("%.79c\n",'-');
+  if( nUpdate==0 ){
+    show_common_info(tid, "checkout:", 1, 0);
+    fossil_print("%-13s None. Already up-to-date\n", "changes:");
+  }else{
+    show_common_info(tid, "updated-to:", 1, 0);
+    fossil_print("%-13s %d file%s modified.\n", "changes:",
+                 nUpdate, nUpdate>1 ? "s" : "");
+  }
 
   /* Report on conflicts
   */
   if( !nochangeFlag ){
+    Stmt q;
+    int nMerge = 0;
+    db_prepare(&q, "SELECT uuid, id FROM vmerge JOIN blob ON merge=rid"
+                   " WHERE id<=0");
+    while( db_step(&q)==SQLITE_ROW ){
+      const char *zLabel = "merge";
+      switch( db_column_int(&q, 1) ){
+        case -1:  zLabel = "cherrypick merge"; break;
+        case -2:  zLabel = "backout merge";    break;
+      }
+      fossil_warning("uncommitted %s against %S.",
+                     zLabel, db_column_text(&q, 0));
+      nMerge++;
+    }
+    db_finalize(&q);
+    
     if( nConflict ){
       if( internalUpdate ){
         internalConflictCnt = nConflict;
@@ -483,6 +509,9 @@ void update_cmd(void){
       fossil_warning("WARNING: %d unmanaged files were overwritten",
                      nOverwrite);
     }
+    if( nMerge ){
+      fossil_warning("WARNING: %d uncommitted prior merges", nMerge);
+    }
   }
   
   /*
@@ -504,6 +533,7 @@ void update_cmd(void){
       db_multi_exec("DELETE FROM vfile WHERE vid!=%d", vid);
     }
     if( !internalUpdate ) undo_finish();
+    if( setmtimeFlag ) vfile_check_signature(vid, CKSIG_SETMTIME);
     db_end_transaction(0);
   }
 }
@@ -570,8 +600,9 @@ int historical_version_of_file(
   const char *revision,    /* The checkin containing the file */
   const char *file,        /* Full treename of the file */
   Blob *content,           /* Put the content here */
-  int *pIsLink,             /* Set to true if file is link. */
+  int *pIsLink,            /* Set to true if file is link. */
   int *pIsExe,             /* Set to true if file is executable */
+  int *pIsBin,             /* Set to true if file is binary */
   int errCode              /* Error code if file not found.  Panic if 0. */
 ){
   Manifest *pManifest;
@@ -592,11 +623,16 @@ int historical_version_of_file(
   if( pManifest ){
     pFile = manifest_file_find(pManifest, file);
     if( pFile ){
+      int rc;
       rid = uuid_to_rid(pFile->zUuid, 0);
       if( pIsExe ) *pIsExe = ( manifest_file_mperm(pFile)==PERM_EXE );
       if( pIsLink ) *pIsLink = ( manifest_file_mperm(pFile)==PERM_LNK );
       manifest_destroy(pManifest);
-      return content_get(rid, content);
+      rc = content_get(rid, content);
+      if( rc && pIsBin ){
+        *pIsBin = looks_like_binary(content);
+      }
+      return rc;
     }
     manifest_destroy(pManifest);
     if( errCode<=0 ){
@@ -665,7 +701,7 @@ void revert_cmd(void){
   }else{
     int vid;
     vid = db_lget_int("checkout", 0);
-    vfile_check_signature(vid, 0, 0);
+    vfile_check_signature(vid, 0);
     db_multi_exec(
       "DELETE FROM vmerge;"
       "INSERT INTO torevert "
@@ -687,7 +723,7 @@ void revert_cmd(void){
     zFile = db_column_text(&q, 0);
     zFull = mprintf("%/%/", g.zLocalRoot, zFile);
     errCode = historical_version_of_file(zRevision, zFile, &record,
-                                         &isLink, &isExe,2);
+                                         &isLink, &isExe, 0, 2);
     if( errCode==2 ){
       if( db_int(0, "SELECT rid FROM vfile WHERE pathname=%Q", zFile)==0 ){
         fossil_print("UNMANAGE: %s\n", zFile);
