@@ -106,7 +106,7 @@ void home_page(void){
   style_header("Home");
   @ <p>This is a stub home-page for the project.
   @ To fill in this page, first go to
-  @ <a href="%s(g.zTop)/setup_config">setup/config</a>
+  @ %z(href("%R/setup_config"))setup/config</a>
   @ and establish a "Project Name".  Then create a
   @ wiki page with that name.  The content of that wiki page
   @ will be displayed in place of this message.</p>
@@ -144,23 +144,21 @@ void wiki_page(void){
     @ <ul>
     { char *zHomePageName = db_get("project-name",0);
       if( zHomePageName ){
-        @ <li> <a href="%s(g.zTop)/wiki?name=%t(zHomePageName)">
+        @ <li> %z(href("%R/wiki?name=%t",zHomePageName))
         @      %h(zHomePageName)</a> wiki home page.</li>
       }
     }
-    @ <li> <a href="%s(g.zTop)/timeline?y=w">Recent changes</a> to wiki
-    @      pages. </li>
-    @ <li> <a href="%s(g.zTop)/wiki_rules">Formatting rules</a> for 
-    @      wiki.</li>
-    @ <li> Use the <a href="%s(g.zTop)/wiki?name=Sandbox">Sandbox</a>
+    @ <li> %z(href("%R/timeline?y=w"))Recent changes</a> to wiki pages.</li>
+    @ <li> %z(href("%R/wiki_rules"))Formatting rules</a> for wiki.</li>
+    @ <li> Use the %z(href("%R/wiki?name=Sandbox"))Sandbox</a>
     @      to experiment.</li>
     if( g.perm.NewWiki ){
-      @ <li>  Create a <a href="%s(g.zTop)/wikinew">new wiki page</a>.</li>
+      @ <li>  Create a %z(href("%R/wikinew"))new wiki page</a>.</li>
       if( g.perm.Write ){
-        @ <li>   Create a <a href="%s(g.zTop)/eventedit">new event</a>.</li>
+        @ <li>   Create a %z(href("%R/eventedit"))new event</a>.</li>
       }
     }
-    @ <li> <a href="%s(g.zTop)/wcontent">List of All Wiki Pages</a>
+    @ <li> %z(href("%R/wcontent"))List of All Wiki Pages</a>
     @      available on this server.</li>
     @ <li> <form method="get" action="%s(g.zTop)/wfind"><div>
     @     Search wiki titles: <input type="text" name="title"/>
@@ -189,23 +187,31 @@ void wiki_page(void){
   }
   if( !g.isHome ){
     if( (rid && g.perm.WrWiki) || (!rid && g.perm.NewWiki) ){
-      style_submenu_element("Edit", "Edit Wiki Page", "%s/wikiedit?name=%T",
-           g.zTop, zPageName);
+      if( db_get_boolean("wysiwyg-wiki", 0) ){
+        style_submenu_element("Edit", "Edit Wiki Page",
+             "%s/wikiedit?name=%T&wysiwyg=1",
+             g.zTop, zPageName);
+      }else{
+        style_submenu_element("Edit", "Edit Wiki Page",
+             "%s/wikiedit?name=%T",
+             g.zTop, zPageName);
+      }
     }
     if( rid && g.perm.ApndWiki && g.perm.Attach ){
       style_submenu_element("Attach", "Add An Attachment",
-           "%s/attachadd?page=%T&amp;from=%s/wiki%%3fname=%T",
+           "%s/attachadd?page=%T&from=%s/wiki%%3fname=%T",
            g.zTop, zPageName, g.zTop, zPageName);
     }
     if( rid && g.perm.ApndWiki ){
       style_submenu_element("Append", "Add A Comment", "%s/wikiappend?name=%T",
            g.zTop, zPageName);
     }
-    if( g.perm.History ){
+    if( g.perm.Hyperlink ){
       style_submenu_element("History", "History", "%s/whistory?name=%T",
            g.zTop, zPageName);
     }
   }
+  style_set_current_page("%s?name=%T", g.zPath, zPageName);
   style_header(zPageName);
   blob_init(&wiki, zBody, -1);
   wiki_convert(&wiki, 0, 0);
@@ -227,8 +233,8 @@ void wiki_page(void){
     }
     cnt++;
     @ <li>
-    if( g.perm.History && g.perm.Read ){
-      @ <a href="%s(g.zTop)/attachview?page=%s(zPageName)&amp;file=%t(zFile)">
+    if( g.perm.Hyperlink && g.perm.Read ){
+      @ %z(href("%R/attachview?page=%T&file=%t",zPageName,zFile))
       @ %h(zFile)</a>
     }else{
       @ %h(zFile)
@@ -236,7 +242,7 @@ void wiki_page(void){
     @ added by %h(zUser) on
     hyperlink_to_date(zDate, ".");
     if( g.perm.WrWiki && g.perm.Attach ){
-      @ [<a href="%s(g.zTop)/attachdelete?page=%s(zPageName)&amp;file=%t(zFile)&amp;from=%s(g.zTop)/wiki%%3fname=%s(zPageName)">delete</a>]
+      @ [%z(href("%R/attachdelete?page=%t&file=%t&from=%R/wiki%%3fname=%f",zPageName,zFile,zPageName))delete</a>]
     }
     @ </li>
   }
@@ -260,13 +266,22 @@ void wikiedit_page(void){
   Blob wiki;
   Manifest *pWiki = 0;
   const char *zPageName;
-  char *zHtmlPageName;
   int n;
   const char *z;
   char *zBody = (char*)P("w");
+  int isWysiwyg = P("wysiwyg")!=0;
 
+  if( P("edit-wysiwyg")!=0 ){ isWysiwyg = 1; zBody = 0; }
+  if( P("edit-markup")!=0 ){ isWysiwyg = 0; zBody = 0; }
   if( zBody ){
-    zBody = mprintf("%s", zBody);
+    if( isWysiwyg ){
+      Blob body;
+      blob_zero(&body);
+      htmlTidy(zBody, &body);
+      zBody = blob_str(&body);
+    }else{
+      zBody = mprintf("%s", zBody);
+    }
   }
   login_check_credentials();
   zPageName = PD("name","");
@@ -338,11 +353,11 @@ void wikiedit_page(void){
   if( zBody==0 ){
     zBody = mprintf("<i>Empty Page</i>");
   }
-  zHtmlPageName = mprintf("Edit: %s", zPageName);
-  style_header(zHtmlPageName);
+  style_set_current_page("%s?name=%T", g.zPath, zPageName);
+  style_header("Edit: %s", zPageName);
+  blob_zero(&wiki);
+  blob_append(&wiki, zBody, -1);
   if( P("preview")!=0 ){
-    blob_zero(&wiki);
-    blob_append(&wiki, zBody, -1);
     @ Preview:<hr />
     wiki_convert(&wiki, 0, 0);
     @ <hr />
@@ -352,18 +367,43 @@ void wikiedit_page(void){
     if( z[0]=='\n' ) n++;
   }
   if( n<20 ) n = 20;
-  if( n>40 ) n = 40;
-  @ <form method="post" action="%s(g.zTop)/wikiedit"><div>
+  if( n>30 ) n = 30;
+  if( !isWysiwyg ){
+    /* Traditional markup-only editing */
+    @ <form method="post" action="%s(g.zTop)/wikiedit"><div>
+    @ <textarea name="w" class="wikiedit" cols="80" 
+    @  rows="%d(n)" wrap="virtual">%h(zBody)</textarea>
+    @ <br />
+    if( db_get_boolean("wysiwyg-wiki", 0) ){
+      @ <input type="submit" name="edit-wysiwyg" value="Wysiwyg Editor"
+      @  onclick='return confirm("Switching to WYSIWYG-mode\nwill erase your markup\nedits. Continue?")' />
+    }
+    @ <input type="submit" name="preview" value="Preview Your Changes" />
+  }else{
+    /* Wysiwyg editing */
+    Blob html, temp;
+    @ <form method="post" action="%s(g.zTop)/wikiedit"
+    @  onsubmit="wysiwygSubmit()"><div>
+    @ <input type="hidden" name="wysiwyg" value="1" />
+    blob_zero(&temp);
+    wiki_convert(&wiki, &temp, 0);
+    blob_zero(&html);
+    htmlTidy(blob_str(&temp), &html);
+    blob_reset(&temp);
+    wysiwygEditor("w", blob_str(&html), 60, n);
+    blob_reset(&html);
+    @ <br />
+    @ <input type="submit" name="edit-markup" value="Markup Editor"
+    @  onclick='return confirm("Switching to markup-mode\nwill erase your WYSIWYG\nedits. Continue?")' />
+  }
+  @ <input type="submit" name="submit" value="Apply These Changes" />
   login_insert_csrf_secret();
   @ <input type="hidden" name="name" value="%h(zPageName)" />
-  @ <textarea name="w" class="wikiedit" cols="80" 
-  @  rows="%d(n)" wrap="virtual">%h(zBody)</textarea>
-  @ <br />
-  @ <input type="submit" name="preview" value="Preview Your Changes" />
-  @ <input type="submit" name="submit" value="Apply These Changes" />
-  @ <input type="submit" name="cancel" value="Cancel" />
+  @ <input type="submit" name="cancel" value="Cancel"
+  @  onclick='confirm("Abandon your changes?")' />
   @ </div></form>
   manifest_destroy(pWiki);
+  blob_reset(&wiki);
   style_footer();
 }
 
@@ -383,7 +423,11 @@ void wikinew_page(void){
   }  
   zName = PD("name","");
   if( zName[0] && wiki_name_is_wellformed((const unsigned char *)zName) ){
-    cgi_redirectf("wikiedit?name=%T", zName);
+    if( db_get_boolean("wysiwyg-wiki", 0) ){
+      cgi_redirectf("wikiedit?name=%T&wysiwyg=1", zName);
+    }else{
+      cgi_redirectf("wikiedit?name=%T", zName);
+    }
   }
   style_header("Create A New Wiki Page");
   @ <p>Rules for wiki page names:</p>
@@ -432,7 +476,6 @@ void wikiappend_page(void){
   int rid = 0;
   int isSandbox;
   const char *zPageName;
-  char *zHtmlPageName;
   const char *zUser;
 
   login_check_credentials();
@@ -507,8 +550,8 @@ void wikiappend_page(void){
     cgi_redirectf("wiki?name=%T", zPageName);
     return;
   }
-  zHtmlPageName = mprintf("Append Comment To: %s", zPageName);
-  style_header(zHtmlPageName);
+  style_set_current_page("%s?name=%T", g.zPath, zPageName);
+  style_header("Append Comment To: %s", zPageName);
   if( P("preview")!=0 ){
     Blob preview;
     blob_zero(&preview);
@@ -546,7 +589,7 @@ static const char *zWikiPageName;
 */
 static void wiki_history_extra(int rid){
   if( db_exists("SELECT 1 FROM tagxref WHERE rid=%d", rid) ){
-    @ <a href="%s(g.zTop)/wdiff?name=%t(zWikiPageName)&amp;a=%d(rid)">[diff]</a>
+    @ %z(href("%R/wdiff?name=%t&a=%d",zWikiPageName,rid))[diff]</a>
   }
 }
 
@@ -562,7 +605,7 @@ void whistory_page(void){
   char *zSQL;
   const char *zPageName;
   login_check_credentials();
-  if( !g.perm.History ){ login_needed(); return; }
+  if( !g.perm.Hyperlink ){ login_needed(); return; }
   zPageName = PD("name","");
   zTitle = mprintf("History Of %s", zPageName);
   style_header(zTitle);
@@ -595,11 +638,11 @@ void wdiff_page(void){
   const char *zPageName;
   Manifest *pW1, *pW2 = 0;
   Blob w1, w2, d;
-  int diffFlags;
+  u64 diffFlags;
 
   login_check_credentials();
   rid1 = atoi(PD("a","0"));
-  if( !g.perm.History ){ login_needed(); return; }
+  if( !g.perm.Hyperlink ){ login_needed(); return; }
   if( rid1==0 ) fossil_redirect_home();
   rid2 = atoi(PD("b","0"));
   zPageName = PD("name","");
@@ -676,9 +719,9 @@ void wcontent_page(void){
     const char *zName = db_column_text(&q, 0);
     int size = db_column_int(&q, 1);
     if( size>0 ){
-      @ <li><a href="%s(g.zTop)/wiki?name=%T(zName)">%h(zName)</a></li>
+      @ <li>%z(href("%R/wiki?name=%T",zName))%h(zName)</a></li>
     }else if( showAll ){
-      @ <li><a href="%s(g.zTop)/wiki?name=%T(zName)"><s>%h(zName)</s></a></li>
+      @ <li>%z(href("%R/wiki?name=%T",zName))<s>%h(zName)</s></a></li>
     }
   }
   db_finalize(&q);
@@ -706,7 +749,7 @@ void wfind_page(void){
 	zTitle);
   while( db_step(&q)==SQLITE_ROW ){
     const char *zName = db_column_text(&q, 0);
-    @ <li><a href="%s(g.zTop)/wiki?name=%T(zName)">%h(zName)</a></li>
+    @ <li>%z(href("%R/wiki?name=%T",zName))%h(zName)</a></li>
   }
   db_finalize(&q);
   @ </ul>
@@ -783,7 +826,7 @@ void wikirules_page(void){
 }
 
 /*
-** Add a new wiki page to the respository.  The page name is
+** Add a new wiki page to the repository.  The page name is
 ** given by the zPageName parameter.  isNew must be true to create
 ** a new page.  If no previous page with the name zPageName exists
 ** and isNew is false, then this routine throws an error.
@@ -873,26 +916,6 @@ int wiki_cmd_commit(char const * zPageName, int isNew, Blob *pContent){
 **        Lists all wiki entries, one per line, ordered
 **        case-insentively by name.
 **
-** TODOs:
-**
-**     %fossil wiki export ?-u ARTIFACT? WikiName ?FILE?
-**
-**        Outputs the selected version of WikiName.
-**
-**     %fossil wiki delete ?-m MESSAGE? WikiName
-**
-**        The same as deleting a file entry, but i don't know if fossil
-**        supports a commit message for Wiki entries.
-**
-**     %fossil wiki ?-u? ?-d? ?-s=[|]? list
-**
-**        Lists the artifact ID and/or Date of last change along with
-**        each entry name, delimited by the -s char.
-**
-**     %fossil wiki diff ?ARTIFACT? ?-f infile[=stdin]? EntryName
-**
-**        Diffs the local copy of a page with a given version (defaulting
-**        to the head version).
 */
 void wiki_cmd(void){
   int n;
