@@ -272,7 +272,7 @@ static int send_delta_parent(
   Blob *pContent,         /* The content of the file to send */
   Blob *pUuid             /* The UUID of the file to send */
 ){
-  static const char *azQuery[] = {
+  static const char *const azQuery[] = {
     "SELECT pid FROM plink x"
     " WHERE cid=%d"
     "   AND NOT EXISTS(SELECT 1 FROM phantom WHERE rid=pid)"
@@ -1048,7 +1048,7 @@ void page_xfer(void){
           }
           seqno++;
         }
-        if( seqno>=max ) seqno = 0;
+        if( seqno>max ) seqno = 0;
         @ clone_seqno %d(seqno)
       }else{
         isClone = 1;
@@ -1208,7 +1208,7 @@ void page_xfer(void){
   if( isClone && nGimme==0 ){
     /* The initial "clone" message from client to server contains no
     ** "gimme" cards. On that initial message, send the client an "igot"
-    ** card for every artifact currently in the respository.  This will
+    ** card for every artifact currently in the repository.  This will
     ** cause the client to create phantoms for all artifacts, which will
     ** in turn make sure that the entire repository is sent efficiently
     ** and expeditiously.
